import unittest

import hcl2

from checkov.terraform.checks.resource.azure.CosmosDBAccountsRestrictedAccess import check
from checkov.common.models.enums import CheckResult


class TestCosmosDBAccountsRestrictedAccess(unittest.TestCase):

    def test_failure_public_access_default(self):
        hcl_res = hcl2.loads("""
           resource "azurerm_cosmosdb_account" "db" {
              name                = "tfex-cosmos-db-${random_integer.ri.result}"
              location            = azurerm_resource_group.rg.location
              resource_group_name = azurerm_resource_group.rg.name
              offer_type          = "Standard"
              kind                = "GlobalDocumentDB"
              is_virtual_network_filter_enabled = true

              enable_automatic_failover = true

              capabilities {
                name = "EnableAggregationPipeline"
              }

              capabilities {
                name = "mongoEnableDocLevelTTL"
              }

              capabilities {
                name = "MongoDBv3.4"
              }

              consistency_policy {
                consistency_level       = "BoundedStaleness"
                max_interval_in_seconds = 10
                max_staleness_prefix    = 200
              }

              geo_location {
                location          = var.failover_location
                failover_priority = 1
              }

              geo_location {
                location          = azurerm_resource_group.rg.location
                failover_priority = 0
              }
            }
            """)
        resource_conf = hcl_res['resource'][0]['azurerm_cosmosdb_account']['db']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_public_access_vn_filter(self):
        hcl_res = hcl2.loads("""
           resource "azurerm_cosmosdb_account" "db" {
              name                = "tfex-cosmos-db-${random_integer.ri.result}"
              location            = azurerm_resource_group.rg.location
              resource_group_name = azurerm_resource_group.rg.name
              offer_type          = "Standard"
              kind                = "GlobalDocumentDB"
              public_network_access_enabled = true
              is_virtual_network_filter_enabled = false

              enable_automatic_failover = true

              capabilities {
                name = "EnableAggregationPipeline"
              }

              capabilities {
                name = "mongoEnableDocLevelTTL"
              }

              capabilities {
                name = "MongoDBv3.4"
              }

              consistency_policy {
                consistency_level       = "BoundedStaleness"
                max_interval_in_seconds = 10
                max_staleness_prefix    = 200
              }

              geo_location {
                location          = var.failover_location
                failover_priority = 1
              }

              geo_location {
                location          = azurerm_resource_group.rg.location
                failover_priority = 0
              }
            }
            """)
        resource_conf = hcl_res['resource'][0]['azurerm_cosmosdb_account']['db']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_vn_filter_false(self):
        hcl_res = hcl2.loads("""
           resource "azurerm_cosmosdb_account" "db" {
              name                = "tfex-cosmos-db-${random_integer.ri.result}"
              location            = azurerm_resource_group.rg.location
              resource_group_name = azurerm_resource_group.rg.name
              offer_type          = "Standard"
              kind                = "GlobalDocumentDB"
              is_virtual_network_filter_enabled = false

              enable_automatic_failover = true

              capabilities {
                name = "EnableAggregationPipeline"
              }

              capabilities {
                name = "mongoEnableDocLevelTTL"
              }

              capabilities {
                name = "MongoDBv3.4"
              }

              consistency_policy {
                consistency_level       = "BoundedStaleness"
                max_interval_in_seconds = 10
                max_staleness_prefix    = 200
              }

              geo_location {
                location          = var.failover_location
                failover_priority = 1
              }

              geo_location {
                location          = azurerm_resource_group.rg.location
                failover_priority = 0
              }
            }
            """)
        resource_conf = hcl_res['resource'][0]['azurerm_cosmosdb_account']['db']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_vm_filter_false_with_vn_rule(self):
        hcl_res = hcl2.loads("""
        resource "azurerm_firewall_network_rule_collection" "example" {
          name                = "testcollection"
          azure_firewall_name = azurerm_firewall.example.name
          resource_group_name = azurerm_resource_group.example.name
          priority            = 100
          action              = "Allow"

          rule {
            name = "testrule"

            source_addresses = [
              "10.0.0.0/16",
            ]

            destination_ports = [
              "53",
            ]

            destination_addresses = [
              "8.8.8.8",
              "8.8.4.4",
            ]

            protocols = [
              "TCP",
              "UDP",
            ]
          }
        }
          resource "azurerm_cosmosdb_account" "db" {
             name                = "tfex-cosmos-db-${random_integer.ri.result}"
             location            = azurerm_resource_group.rg.location
             resource_group_name = azurerm_resource_group.rg.name
             offer_type          = "Standard"
             kind                = "GlobalDocumentDB"
             is_virtual_network_filter_enabled = false
             virtual_network_rule = azurerm_firewall_network_rule_collection.example

             enable_automatic_failover = true

             capabilities {
               name = "EnableAggregationPipeline"
             }

             capabilities {
               name = "mongoEnableDocLevelTTL"
             }

             capabilities {
               name = "MongoDBv3.4"
             }

             consistency_policy {
               consistency_level       = "BoundedStaleness"
               max_interval_in_seconds = 10
               max_staleness_prefix    = 200
             }

             geo_location {
               location          = var.failover_location
               failover_priority = 1
             }

             geo_location {
               location          = azurerm_resource_group.rg.location
               failover_priority = 0
             }
           }
               """)
        resource_conf = hcl_res['resource'][1]['azurerm_cosmosdb_account']['db']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_public_access_with_ip_range(self):
        hcl_res = hcl2.loads("""
           resource "azurerm_cosmosdb_account" "db" {
              name                = "tfex-cosmos-db-${random_integer.ri.result}"
              location            = azurerm_resource_group.rg.location
              resource_group_name = azurerm_resource_group.rg.name
              offer_type          = "Standard"
              kind                = "GlobalDocumentDB"
              public_network_access_enabled = true
              
              ip_range_filter = ["192.0.0.1"]

              enable_automatic_failover = true

              capabilities {
                name = "EnableAggregationPipeline"
              }

              capabilities {
                name = "mongoEnableDocLevelTTL"
              }

              capabilities {
                name = "MongoDBv3.4"
              }

              consistency_policy {
                consistency_level       = "BoundedStaleness"
                max_interval_in_seconds = 10
                max_staleness_prefix    = 200
              }

              geo_location {
                location          = var.failover_location
                failover_priority = 1
              }

              geo_location {
                location          = azurerm_resource_group.rg.location
                failover_priority = 0
              }
            }
            """)
        resource_conf = hcl_res['resource'][0]['azurerm_cosmosdb_account']['db']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success_no_public_access(self):
        hcl_res = hcl2.loads("""
           resource "azurerm_cosmosdb_account" "db" {
              name                = "tfex-cosmos-db-${random_integer.ri.result}"
              location            = azurerm_resource_group.rg.location
              resource_group_name = azurerm_resource_group.rg.name
              offer_type          = "Standard"
              kind                = "GlobalDocumentDB"
              public_network_access_enabled = false

              enable_automatic_failover = true

              capabilities {
                name = "EnableAggregationPipeline"
              }

              capabilities {
                name = "mongoEnableDocLevelTTL"
              }

              capabilities {
                name = "MongoDBv3.4"
              }

              consistency_policy {
                consistency_level       = "BoundedStaleness"
                max_interval_in_seconds = 10
                max_staleness_prefix    = 200
              }

              geo_location {
                location          = var.failover_location
                failover_priority = 1
              }

              geo_location {
                location          = azurerm_resource_group.rg.location
                failover_priority = 0
              }
            }
            """)
        resource_conf = hcl_res['resource'][0]['azurerm_cosmosdb_account']['db']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_vm_filter_with_vm_rule(self):
        hcl_res = hcl2.loads("""
        resource "azurerm_firewall_network_rule_collection" "example" {
          name                = "testcollection"
          azure_firewall_name = azurerm_firewall.example.name
          resource_group_name = azurerm_resource_group.example.name
          priority            = 100
          action              = "Allow"
        
          rule {
            name = "testrule"
        
            source_addresses = [
              "10.0.0.0/16",
            ]
        
            destination_ports = [
              "53",
            ]
        
            destination_addresses = [
              "8.8.8.8",
              "8.8.4.4",
            ]
        
            protocols = [
              "TCP",
              "UDP",
            ]
          }
        }
              resource "azurerm_cosmosdb_account" "db" {
                 name                = "tfex-cosmos-db-${random_integer.ri.result}"
                 location            = azurerm_resource_group.rg.location
                 resource_group_name = azurerm_resource_group.rg.name
                 offer_type          = "Standard"
                 kind                = "GlobalDocumentDB"
                 is_virtual_network_filter_enabled = true
                 virtual_network_rule = azurerm_firewall_network_rule_collection.example

                 enable_automatic_failover = true

                 capabilities {
                   name = "EnableAggregationPipeline"
                 }

                 capabilities {
                   name = "mongoEnableDocLevelTTL"
                 }

                 capabilities {
                   name = "MongoDBv3.4"
                 }

                 consistency_policy {
                   consistency_level       = "BoundedStaleness"
                   max_interval_in_seconds = 10
                   max_staleness_prefix    = 200
                 }

                 geo_location {
                   location          = var.failover_location
                   failover_priority = 1
                 }

                 geo_location {
                   location          = azurerm_resource_group.rg.location
                   failover_priority = 0
                 }
               }
               """)
        resource_conf = hcl_res['resource'][1]['azurerm_cosmosdb_account']['db']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_no_public_access_with_vn_filter(self):
        hcl_res = hcl2.loads("""
        resource "azurerm_firewall_network_rule_collection" "example" {
          name                = "testcollection"
          azure_firewall_name = azurerm_firewall.example.name
          resource_group_name = azurerm_resource_group.example.name
          priority            = 100
          action              = "Allow"

          rule {
            name = "testrule"

            source_addresses = [
              "10.0.0.0/16",
            ]

            destination_ports = [
              "53",
            ]

            destination_addresses = [
              "8.8.8.8",
              "8.8.4.4",
            ]

            protocols = [
              "TCP",
              "UDP",
            ]
          }
        }
              resource "azurerm_cosmosdb_account" "db" {
                 name                = "tfex-cosmos-db-${random_integer.ri.result}"
                 location            = azurerm_resource_group.rg.location
                 resource_group_name = azurerm_resource_group.rg.name
                 offer_type          = "Standard"
                 kind                = "GlobalDocumentDB"
                 public_network_access_enabled = false
                 is_virtual_network_filter_enabled = true
                 virtual_network_rule = azurerm_firewall_network_rule_collection.example

                 enable_automatic_failover = true

                 capabilities {
                   name = "EnableAggregationPipeline"
                 }

                 capabilities {
                   name = "mongoEnableDocLevelTTL"
                 }

                 capabilities {
                   name = "MongoDBv3.4"
                 }

                 consistency_policy {
                   consistency_level       = "BoundedStaleness"
                   max_interval_in_seconds = 10
                   max_staleness_prefix    = 200
                 }

                 geo_location {
                   location          = var.failover_location
                   failover_priority = 1
                 }

                 geo_location {
                   location          = azurerm_resource_group.rg.location
                   failover_priority = 0
                 }
               }
               """)
        resource_conf = hcl_res['resource'][1]['azurerm_cosmosdb_account']['db']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_vn_filter_ip_range(self):
        hcl_res = hcl2.loads("""
        resource "azurerm_firewall_network_rule_collection" "example" {
          name                = "testcollection"
          azure_firewall_name = azurerm_firewall.example.name
          resource_group_name = azurerm_resource_group.example.name
          priority            = 100
          action              = "Allow"

          rule {
            name = "testrule"

            source_addresses = [
              "10.0.0.0/16",
            ]

            destination_ports = [
              "53",
            ]

            destination_addresses = [
              "8.8.8.8",
              "8.8.4.4",
            ]

            protocols = [
              "TCP",
              "UDP",
            ]
          }
        }
              resource "azurerm_cosmosdb_account" "db" {
                 name                = "tfex-cosmos-db-${random_integer.ri.result}"
                 location            = azurerm_resource_group.rg.location
                 resource_group_name = azurerm_resource_group.rg.name
                 offer_type          = "Standard"
                 kind                = "GlobalDocumentDB"
                 is_virtual_network_filter_enabled = true
                 ip_range_filter = ["192.0.0.1"]

                 enable_automatic_failover = true

                 capabilities {
                   name = "EnableAggregationPipeline"
                 }

                 capabilities {
                   name = "mongoEnableDocLevelTTL"
                 }

                 capabilities {
                   name = "MongoDBv3.4"
                 }

                 consistency_policy {
                   consistency_level       = "BoundedStaleness"
                   max_interval_in_seconds = 10
                   max_staleness_prefix    = 200
                 }

                 geo_location {
                   location          = var.failover_location
                   failover_priority = 1
                 }

                 geo_location {
                   location          = azurerm_resource_group.rg.location
                   failover_priority = 0
                 }
               }
               """)
        resource_conf = hcl_res['resource'][1]['azurerm_cosmosdb_account']['db']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()




def test_reduce_scan_reports_secrets(report):
    from checkov.common.bridgecrew.wrapper import reduce_scan_reports
    from checkov.common.typing import _ReducedScanReportCheck, _ReducedScanReport
    from checkov.common.bridgecrew.check_type import CheckType

    reduced_report: _ReducedScanReport = reduce_scan_reports([report])[CheckType.SECRETS]

    checks: _ReducedScanReportCheck = reduced_report["checks"]
    all_checks = checks["passed_checks"] + checks["failed_checks"] + checks["skipped_checks"]

    assert all('validation_status' in check.keys() for check in all_checks)


def test_reduce_scan_reports(report):
    from checkov.common.bridgecrew.wrapper import reduce_scan_reports
    from checkov.common.typing import _ReducedScanReportCheck, _ReducedScanReport
    from checkov.common.bridgecrew.check_type import CheckType

    report.check_type = CheckType.GITHUB_ACTIONS
    reduced_report: _ReducedScanReport = reduce_scan_reports([report])[CheckType.GITHUB_ACTIONS]

    checks: _ReducedScanReportCheck = reduced_report["checks"]
    all_checks = checks["passed_checks"] + checks["failed_checks"] + checks["skipped_checks"]

    reduced_keys = ('check_id', 'check_result', 'resource', 'file_path', 'file_line_range')

    assert all(reduced_key in check.keys() for check in all_checks for reduced_key in reduced_keys)
    assert all('validation_status' not in check.keys() for check in all_checks)



"""Process-global verified-sources registry: chokepoint ↔ loader bridge."""
from __future__ import annotations

import logging
import os
from typing import Dict, Iterable, List, Optional

from .enforce import verify_external_checks_dirs
from .keys import load_public_keys


logger = logging.getLogger(__name__)


# None = inactive; empty dict = active but no .py files found.
_verified_sources: Optional[Dict[str, bytes]] = None


def is_verification_active() -> bool:
    return _verified_sources is not None


def verify_and_register(
    dirs: Iterable[str],
    public_key_paths: List[str],
) -> None:
    """No-op when ``public_key_paths`` is empty. Raises on verification failure.

    On failure the registry is cleared (set back to inactive).
    """
    global _verified_sources

    if not public_key_paths:
        return

    keys = load_public_keys(public_key_paths)
    try:
        verified = verify_external_checks_dirs(dirs, keys)
    except BaseException:
        _verified_sources = None
        raise
    _verified_sources = {
        os.path.normpath(os.path.realpath(p)): b for p, b in verified.items()
    }
    logger.info("external-checks verification registered: %d files", len(_verified_sources))


def get_verified_sources_for_directory(directory: str) -> "Optional[Dict[str, bytes]]":
    """Return None when inactive; else the subset under ``directory``."""
    if _verified_sources is None:
        return None
    directory_real = os.path.normpath(os.path.realpath(directory))
    prefix = directory_real + os.sep
    return {
        path: source_bytes
        for path, source_bytes in _verified_sources.items()
        if path == directory_real or path.startswith(prefix)
    }


def get_all_verified_sources() -> "Optional[Dict[str, bytes]]":
    """Return the FULL registry snapshot, or ``None`` when inactive.

    Used by the loader to serve cross-directory transitive imports
    from verified bytes (otherwise a verified check in dir-A that does
    ``import shared_helper`` from dir-B would fall through to the
    unverified on-disk file).
    """
    if _verified_sources is None:
        return None
    return dict(_verified_sources)


def reset_for_tests() -> None:
    global _verified_sources
    _verified_sources = None


__all__ = [
    "get_all_verified_sources",
    "get_verified_sources_for_directory",
    "is_verification_active",
    "reset_for_tests",
    "verify_and_register",
]



from __future__ import annotations

import hashlib
import shutil
import subprocess
from pathlib import Path

import pytest
from ecdsa import NIST256p
from ecdsa.util import sigdecode_der, sigencode_der

from checkov.common.external_checks.verification import (
    VerificationResult,
    load_public_keys,
    verify_file,
)
from checkov.common.external_checks.verification.trailer_format import (
    TRAILER_PREFIX,
)


_SECP256R1_N = NIST256p.order


def _sign(priv, body: bytes, hashfunc=hashlib.sha256) -> bytes:
    """Produce a DER ECDSA signature over ``body`` using ``hashfunc``."""
    return priv.sign_deterministic(body, hashfunc=hashfunc, sigencode=sigencode_der)


# --------------------------------------------------------------------------
# 1. Python ``ecdsa`` library — sanity baseline.
# --------------------------------------------------------------------------


def test_python_ecdsa_signature_verifies(
    tmp_path: Path, priv_a, key_a_pub_pem: bytes, make_trailer,
):
    """A signature produced via the standard ``sign_deterministic`` path verifies.

    Baseline interop test — if this fails, nothing else can succeed.
    """
    body = b"def check():\n    pass\n"
    target = tmp_path / "py_signed.py"
    target.write_bytes(make_trailer(body, priv_a))

    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    result, signed = verify_file(str(target), load_public_keys([str(key_path)]))
    assert result == VerificationResult.OK
    assert signed == body


# --------------------------------------------------------------------------
# 2. OpenSSL CLI — the docs recipe MUST work verbatim.
# --------------------------------------------------------------------------


@pytest.mark.skipif(shutil.which("openssl") is None, reason="openssl CLI not installed")
def test_openssl_dgst_sign_recipe_from_docs_verifies(tmp_path: Path):
    """The exact OpenSSL signing recipe from the customer-facing docs
    must produce a signature the verifier accepts.
    """
    priv_pem = tmp_path / "priv.pem"
    pub_pem = tmp_path / "pub.pem"
    subprocess.run(
        ["openssl", "ecparam", "-name", "prime256v1", "-genkey", "-noout", "-out", str(priv_pem)],
        check=True, capture_output=True,
    )
    subprocess.run(
        ["openssl", "ec", "-in", str(priv_pem), "-pubout", "-out", str(pub_pem)],
        check=True, capture_output=True,
    )

    body = b"# openssl-signed check\nID = 'CKV_OSSL_1'\n"
    target = tmp_path / "ossl_signed.py"
    target.write_bytes(body)

    sig_proc = subprocess.run(
        ["openssl", "dgst", "-sha256", "-sign", str(priv_pem), str(target)],
        check=True, capture_output=True,
    )
    sig_der = sig_proc.stdout
    hex_payload = sig_der.hex().encode("ascii")

    with open(target, "ab") as f:
        f.write(TRAILER_PREFIX + hex_payload + b"\n")

    result, signed = verify_file(str(target), load_public_keys([str(pub_pem)]))
    assert result == VerificationResult.OK, (
        f"openssl-signed file did not verify; got {result!r}"
    )
    assert signed == body, (
        "verifier returned the wrong bytes — the trailer-stripping step "
        "or the openssl signing step is producing the wrong payload"
    )


# --------------------------------------------------------------------------
# 3. High-S (KMS-style) signatures.
#
# AWS KMS, openssl dgst, and several other production signers emit
# high-S roughly 50% of the time. A verifier that silently rejected
# high-S would break ~half of all customer signatures with no apparent
# pattern.
# --------------------------------------------------------------------------


def _flip_to_high_s(sig_der: bytes) -> bytes:
    """Return a DER signature with ``s`` flipped to its high-S equivalent."""
    r, s = sigdecode_der(sig_der, _SECP256R1_N)
    if s > _SECP256R1_N // 2:
        return sig_der
    high_s = _SECP256R1_N - s
    return sigencode_der(r, high_s, _SECP256R1_N)


def _flip_to_low_s(sig_der: bytes) -> bytes:
    """Return a DER signature with ``s`` flipped to its low-S equivalent."""
    r, s = sigdecode_der(sig_der, _SECP256R1_N)
    if s <= _SECP256R1_N // 2:
        return sig_der
    return sigencode_der(r, _SECP256R1_N - s, _SECP256R1_N)


def test_high_s_signature_verifies(
    tmp_path: Path, priv_a, key_a_pub_pem: bytes,
):
    """A signature with ``s > n/2`` must verify (the AWS-KMS scenario)."""
    body = b"def check():\n    pass\n"
    raw_sig = _sign(priv_a, body)
    high_s_sig = _flip_to_high_s(raw_sig)
    _r, s = sigdecode_der(high_s_sig, _SECP256R1_N)
    assert s > _SECP256R1_N // 2, "test helper failed to produce a high-S sig"

    target = tmp_path / "high_s.py"
    target.write_bytes(body + TRAILER_PREFIX + high_s_sig.hex().encode("ascii") + b"\n")

    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    result, signed = verify_file(str(target), load_public_keys([str(key_path)]))
    assert result == VerificationResult.OK, (
        f"high-S signature was rejected; verifier got {result!r}. "
        f"This would break ~50% of AWS-KMS-signed customer files. "
        f"Check the low-S enforcement comment in verifier.py."
    )
    assert signed == body


def test_low_s_signature_verifies(
    tmp_path: Path, priv_a, key_a_pub_pem: bytes,
):
    """Companion to the high-S test: low-S signatures must also verify."""
    body = b"def check():\n    pass\n"
    raw_sig = _sign(priv_a, body)
    low_s_sig = _flip_to_low_s(raw_sig)
    _r, s = sigdecode_der(low_s_sig, _SECP256R1_N)
    assert s <= _SECP256R1_N // 2, "test helper failed to produce a low-S sig"

    target = tmp_path / "low_s.py"
    target.write_bytes(body + TRAILER_PREFIX + low_s_sig.hex().encode("ascii") + b"\n")

    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    result, _ = verify_file(str(target), load_public_keys([str(key_path)]))
    assert result == VerificationResult.OK


def test_high_s_and_low_s_are_equivalent(
    tmp_path: Path, priv_a, key_a_pub_pem: bytes,
):
    """High-S and low-S forms of the same signature both verify the same payload."""
    body = b"# original\n"
    raw_sig = _sign(priv_a, body)
    high_s = _flip_to_high_s(raw_sig)
    low_s = _flip_to_low_s(raw_sig)

    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    keys = load_public_keys([str(key_path)])

    for name, sig in (("high_s", high_s), ("low_s", low_s)):
        target = tmp_path / f"{name}.py"
        target.write_bytes(body + TRAILER_PREFIX + sig.hex().encode("ascii") + b"\n")
        result, signed = verify_file(str(target), keys)
        assert result == VerificationResult.OK, f"{name} failed: {result!r}"
        assert signed == body


# --------------------------------------------------------------------------
# 4. Edge-of-range DER signatures: very short and very long but legal.
#
# DER-encoded P-256 sigs vary in length between ~63 and 72 bytes
# depending on leading-zero stripping in r and s.
# --------------------------------------------------------------------------


def test_full_length_signature_verifies(
    tmp_path: Path, priv_a, key_a_pub_pem: bytes,
):
    """A 70-72-byte (full-length r and s) signature verifies."""
    sig = b""
    final_body = b""
    for i in range(200):
        candidate_body = f"x{i}\n".encode("ascii")
        candidate_sig = _sign(priv_a, candidate_body)
        if len(candidate_sig) >= 70:
            sig = candidate_sig
            final_body = candidate_body
            break
    assert len(sig) >= 70, "could not produce a 70-byte sig in 200 tries"

    target = tmp_path / "full_length.py"
    target.write_bytes(final_body + TRAILER_PREFIX + sig.hex().encode("ascii") + b"\n")

    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    result, signed = verify_file(str(target), load_public_keys([str(key_path)]))
    assert result == VerificationResult.OK
    assert signed == final_body


def test_short_signature_verifies(
    tmp_path: Path, priv_a, key_a_pub_pem: bytes,
):
    """A short signature (≤ 68 bytes — leading zeros stripped) verifies."""
    sig = b""
    final_body = b""
    for i in range(500):
        candidate_body = f"y{i}\n".encode("ascii")
        candidate_sig = _sign(priv_a, candidate_body)
        if len(candidate_sig) <= 68:
            sig = candidate_sig
            final_body = candidate_body
            break
    if not final_body:
        pytest.skip("could not produce a short sig in 500 tries (rare)")

    target = tmp_path / "short.py"
    target.write_bytes(final_body + TRAILER_PREFIX + sig.hex().encode("ascii") + b"\n")

    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    result, signed = verify_file(str(target), load_public_keys([str(key_path)]))
    assert result == VerificationResult.OK
    assert signed == final_body


# --------------------------------------------------------------------------
# 5. PEM key shape variants.
# --------------------------------------------------------------------------


@pytest.mark.skipif(shutil.which("openssl") is None, reason="openssl CLI not installed")
def test_openssl_generated_pem_loads(tmp_path: Path):
    """A PEM produced by ``openssl ec -in priv.pem -pubout`` loads correctly."""
    priv_pem = tmp_path / "priv.pem"
    pub_pem = tmp_path / "pub.pem"
    subprocess.run(
        ["openssl", "ecparam", "-name", "prime256v1", "-genkey", "-noout", "-out", str(priv_pem)],
        check=True, capture_output=True,
    )
    subprocess.run(
        ["openssl", "ec", "-in", str(priv_pem), "-pubout", "-out", str(pub_pem)],
        check=True, capture_output=True,
    )
    keys = load_public_keys([str(pub_pem)])
    assert len(keys) == 1
    assert keys[0].source_path == str(pub_pem)


def test_ecdsa_generated_pem_loads(tmp_path: Path, priv_a):
    """A PEM produced via ``priv.get_verifying_key().to_pem()`` loads."""
    pub_pem = priv_a.get_verifying_key().to_pem()
    pub_path = tmp_path / "pub.pem"
    pub_path.write_bytes(pub_pem)
    keys = load_public_keys([str(pub_path)])
    assert len(keys) == 1


# --------------------------------------------------------------------------
# 6. Multi-key configurations (rotation scenario).
# --------------------------------------------------------------------------


def test_signature_verifies_against_second_key_in_rotation_list(
    tmp_path: Path, priv_a, key_a_pub_pem: bytes, key_b_pub_pem: bytes,
):
    """A signature by ``priv_a`` verifies when keys are configured as ``[b, a]``."""
    body = b"# rotation scenario\n"
    sig = _sign(priv_a, body)
    target = tmp_path / "rotated.py"
    target.write_bytes(body + TRAILER_PREFIX + sig.hex().encode("ascii") + b"\n")

    key_a_path = tmp_path / "a.pem"
    key_a_path.write_bytes(key_a_pub_pem)
    key_b_path = tmp_path / "b.pem"
    key_b_path.write_bytes(key_b_pub_pem)
    keys = load_public_keys([str(key_b_path), str(key_a_path)])

    result, _ = verify_file(str(target), keys)
    assert result == VerificationResult.OK


# --------------------------------------------------------------------------
# 7. Deterministic interop: any valid DER (r, s) encoding verifies.
# --------------------------------------------------------------------------


def test_manually_encoded_dss_signature_verifies(
    tmp_path: Path, priv_a, key_a_pub_pem: bytes,
):
    """A signature constructed from ``sigencode_der(r, s, n)`` verifies.

    Pins that the verifier accepts ANY valid DER (r, s) encoding, not
    just whatever encoding the producing library happens to emit.
    """
    body = b"# DER round-trip\n"
    raw_sig = _sign(priv_a, body)
    r, s = sigdecode_der(raw_sig, _SECP256R1_N)
    re_encoded = sigencode_der(r, s, _SECP256R1_N)

    target = tmp_path / "reencoded.py"
    target.write_bytes(body + TRAILER_PREFIX + re_encoded.hex().encode("ascii") + b"\n")

    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    result, signed = verify_file(str(target), load_public_keys([str(key_path)]))
    assert result == VerificationResult.OK
    assert signed == body


# --------------------------------------------------------------------------
# 8. Signing-input-byte sensitivity.
# --------------------------------------------------------------------------


def test_signing_input_must_match_on_disk_pretrailer_bytes(
    tmp_path: Path, priv_a, key_a_pub_pem: bytes,
):
    """Signature is invalid if signer hashed different bytes than disk."""
    on_disk_body = b"# real body\n"
    fake_signed_payload = b"# different body the signer thought it was signing\n"
    sig = _sign(priv_a, fake_signed_payload)

    target = tmp_path / "mismatched.py"
    target.write_bytes(on_disk_body + TRAILER_PREFIX + sig.hex().encode("ascii") + b"\n")

    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    result, _ = verify_file(str(target), load_public_keys([str(key_path)]))
    assert result == VerificationResult.UNKNOWN_KEY, (
        "verifier accepted a signature over a different payload than the on-disk body"
    )


def test_signing_input_excludes_trailer_line_and_its_newline(
    tmp_path: Path, priv_a, key_a_pub_pem: bytes,
):
    """The bytes the verifier hashes are EXACTLY ``file_bytes[:trailer_start]``.

    Specifically:
    * The terminating ``\\n`` of the body IS included in the signed bytes.
    * The trailer line itself and its terminating ``\\n`` are NOT.
    """
    body = b"line one\nline two\n"
    sig = _sign(priv_a, body)
    target = tmp_path / "exact.py"
    target.write_bytes(body + TRAILER_PREFIX + sig.hex().encode("ascii") + b"\n")

    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    result, signed = verify_file(str(target), load_public_keys([str(key_path)]))
    assert result == VerificationResult.OK
    assert signed == body, (
        f"verifier returned the wrong signed-bytes slice: expected "
        f"{body!r}, got {signed!r}"
    )


# --------------------------------------------------------------------------
# 9. JWS / IEEE P1363 raw ``r||s`` signatures MUST be rejected.
#
# JWT/JWS libraries (PyJWT, jose, java-jwt, …) emit 64-byte flat
# ``r||s`` concatenations, NOT DER. The hex length is 128 chars — INSIDE
# our [126, 144] length guard — so the parser will let the trailer line
# through, and the DER-decode step is the only thing that distinguishes
# the two encodings. The rejection MUST happen via ``BAD_SIGNATURE``
# (well-formed trailer, malformed DER), not ``UNKNOWN_KEY``.
# --------------------------------------------------------------------------


def test_jose_raw_r_s_signature_rejected_as_bad_signature(
    tmp_path: Path, priv_a, key_a_pub_pem: bytes,
):
    """A 64-byte raw ``r||s`` signature (the JWS / IEEE P1363 form) is rejected."""
    body = b"def check():\n    return 'ok'\n"
    der_sig = _sign(priv_a, body)
    r, s = sigdecode_der(der_sig, _SECP256R1_N)
    raw_sig = r.to_bytes(32, "big") + s.to_bytes(32, "big")
    assert len(raw_sig) == 64, "test helper failed to produce a 64-byte raw sig"

    target = tmp_path / "jose_style.py"
    target.write_bytes(body + TRAILER_PREFIX + raw_sig.hex().encode("ascii") + b"\n")

    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    keys = load_public_keys([str(key_path)])

    result, _ = verify_file(str(target), keys)
    assert result == VerificationResult.BAD_SIGNATURE, (
        f"raw r||s (JWS/IEEE-P1363) signature was not rejected as "
        f"BAD_SIGNATURE; got {result!r}. The verifier accepts ONLY DER."
    )


# --------------------------------------------------------------------------
# 10. Non-SPKI / wrong-curve / wrong-algorithm key shapes are rejected.
#
# Unlike the ``cryptography`` library, ``ecdsa.VerifyingKey.from_pem``
# does NOT cross-check the PEM banner against the body — it only parses
# the body. So a PEM with the legacy ``-----BEGIN EC PUBLIC KEY-----``
# banner wrapped around a valid SPKI body is accepted (the key material
# is the same — no security impact). What MUST still be rejected is any
# key whose curve is not NIST256p.
# --------------------------------------------------------------------------


def test_legacy_banner_with_spki_body_is_accepted(tmp_path: Path, priv_a):
    """A PEM whose banner says ``EC PUBLIC KEY`` but whose body is SPKI is
    accepted — the underlying key material is identical.

    Documents that ``ecdsa.VerifyingKey.from_pem`` is banner-tolerant,
    in contrast to ``cryptography.serialization.load_pem_public_key``.
    """
    spki_pem = priv_a.get_verifying_key().to_pem()
    legacy_pem = (
        spki_pem
        .replace(b"-----BEGIN PUBLIC KEY-----", b"-----BEGIN EC PUBLIC KEY-----")
        .replace(b"-----END PUBLIC KEY-----", b"-----END EC PUBLIC KEY-----")
    )
    pub_path = tmp_path / "legacy_banner.pem"
    pub_path.write_bytes(legacy_pem)
    keys = load_public_keys([str(pub_path)])
    assert len(keys) == 1


def test_garbage_pem_rejected_with_clear_error(tmp_path: Path):
    """A PEM-looking file with no valid SPKI body is rejected."""
    from checkov.common.external_checks.verification.errors import (
        SignatureVerificationError,
    )
    pub_path = tmp_path / "garbage.pem"
    pub_path.write_bytes(
        b"-----BEGIN PUBLIC KEY-----\nnot-base64-at-all\n-----END PUBLIC KEY-----\n"
    )
    with pytest.raises(SignatureVerificationError, match="unsupported key format"):
        load_public_keys([str(pub_path)])


# --------------------------------------------------------------------------
# 11. Wrong hash algorithm on the signing side.
#
# The verifier hardcodes SHA-256. A signature computed with SHA-384 or
# SHA-512 is structurally valid (DER) but cryptographically invalid for
# the payload-under-SHA-256, so the verifier must reject with
# ``UNKNOWN_KEY``.
# --------------------------------------------------------------------------


def test_sha384_signed_file_is_rejected(
    tmp_path: Path, priv_a, key_a_pub_pem: bytes,
):
    """A signature computed with SHA-384 over the payload is rejected."""
    body = b"# wrong-hash signer\n"
    sha384_sig = _sign(priv_a, body, hashfunc=hashlib.sha384)

    target = tmp_path / "sha384.py"
    target.write_bytes(body + TRAILER_PREFIX + sha384_sig.hex().encode("ascii") + b"\n")

    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    result, _ = verify_file(str(target), load_public_keys([str(key_path)]))
    assert result == VerificationResult.UNKNOWN_KEY, (
        f"SHA-384-signed file produced {result!r}; expected UNKNOWN_KEY. "
        f"If the verifier ever silently accepts non-SHA-256 hashes, "
        f"that's a primitive-pinning regression."
    )


def test_sha512_signed_file_is_rejected(
    tmp_path: Path, priv_a, key_a_pub_pem: bytes,
):
    """Companion to the SHA-384 test — SHA-512 is also rejected."""
    body = b"# wrong-hash signer (sha512)\n"
    sha512_sig = _sign(priv_a, body, hashfunc=hashlib.sha512)

    target = tmp_path / "sha512.py"
    target.write_bytes(body + TRAILER_PREFIX + sha512_sig.hex().encode("ascii") + b"\n")

    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    result, _ = verify_file(str(target), load_public_keys([str(key_path)]))
    assert result == VerificationResult.UNKNOWN_KEY


# --------------------------------------------------------------------------
# 12. Pre-hashed signing path (operator misconfiguration).
# --------------------------------------------------------------------------


def test_pre_hashed_signature_rejected(
    tmp_path: Path, priv_a, key_a_pub_pem: bytes,
):
    """A signature over ``sha256(body)`` (instead of ``body``) is rejected.

    The classic double-hash signing mistake.
    """
    body = b"# pre-hashed signing mistake\n"
    digest = hashlib.sha256(body).digest()
    sig = _sign(priv_a, digest)

    target = tmp_path / "double_hashed.py"
    target.write_bytes(body + TRAILER_PREFIX + sig.hex().encode("ascii") + b"\n")

    key_path = tmp_path / "key.pem"
    key_path.write_bytes(key_a_pub_pem)
    result, _ = verify_file(str(target), load_public_keys([str(key_path)]))
    assert result == VerificationResult.UNKNOWN_KEY, (
        f"double-hash signing mistake produced {result!r}; expected "
        f"UNKNOWN_KEY. Customer support tickets for this scenario need "
        f"the dedicated diagnostic."
    )



import subprocess

from checkov.kustomize.runner import Runner


def kustomize_exists() -> bool:
    try:
        subprocess.run([Runner.kustomize_command, "version"], check=True, stdout=subprocess.PIPE)
    except Exception:
        return False
    return True



import unittest

import hcl2

from checkov.terraform.checks.resource.aws.AuroraEncryption import check
from checkov.common.models.enums import CheckResult


class TestAuroraEncryption(unittest.TestCase):

    def test_success(self):
        hcl_res = hcl2.loads("""
        resource "aws_rds_cluster" "test" {
            cluster_identifier      = "aurora-cluster-demo"
            engine                  = "aurora-mysql"
            engine_version          = "5.7.mysql_aurora.2.03.2"
            availability_zones      = ["us-west-2a", "us-west-2b", "us-west-2c"]
            database_name           = "mydb"
            master_username         = "foo"
            master_password         = "bar"
            backup_retention_period = 5
            preferred_backup_window = "07:00-09:00"
            storage_encrypted       = true
        }
        """)
        resource_conf = hcl_res['resource'][0]['aws_rds_cluster']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_serverless_default(self):
        hcl_res = hcl2.loads("""
        resource "aws_rds_cluster" "test" {
            cluster_identifier      = "aurora-cluster-demo"
            availability_zones      = ["us-west-2a", "us-west-2b", "us-west-2c"]
            database_name           = "mydb"
            master_username         = "foo"
            master_password         = "bar"
            backup_retention_period = 5
            preferred_backup_window = "07:00-09:00"
            engine_mode             = "serverless"
        }
        """)
        resource_conf = hcl_res['resource'][0]['aws_rds_cluster']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    # If engine_mode is serverless then storage_encrypted should be ignored
    def test_success_serverless_enc_off(self):
        hcl_res = hcl2.loads("""
        resource "aws_rds_cluster" "test" {
            cluster_identifier      = "aurora-cluster-demo"
            availability_zones      = ["us-west-2a", "us-west-2b", "us-west-2c"]
            database_name           = "mydb"
            master_username         = "foo"
            master_password         = "bar"
            backup_retention_period = 5
            preferred_backup_window = "07:00-09:00"
            engine_mode             = "serverless"
            storage_encrypted       = false
        }
        """)
        resource_conf = hcl_res['resource'][0]['aws_rds_cluster']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_failure_explicit(self):
        hcl_res = hcl2.loads("""
        resource "aws_rds_cluster" "test" {
            cluster_identifier      = "aurora-cluster-demo"
            engine                  = "aurora-mysql"
            engine_version          = "5.7.mysql_aurora.2.03.2"
            availability_zones      = ["us-west-2a", "us-west-2b", "us-west-2c"]
            database_name           = "mydb"
            master_username         = "foo"
            master_password         = "bar"
            backup_retention_period = 5
            preferred_backup_window = "07:00-09:00"
            storage_encrypted       = false
        }
        """)
        resource_conf = hcl_res['resource'][0]['aws_rds_cluster']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_default(self):
        hcl_res = hcl2.loads("""
        resource "aws_rds_cluster" "test" {
            cluster_identifier      = "aurora-cluster-demo"
            engine                  = "aurora-mysql"
            engine_version          = "5.7.mysql_aurora.2.03.2"
            availability_zones      = ["us-west-2a", "us-west-2b", "us-west-2c"]
            database_name           = "mydb"
            master_username         = "foo"
            master_password         = "bar"
            backup_retention_period = 5
            preferred_backup_window = "07:00-09:00"
        }
        """)
        resource_conf = hcl_res['resource'][0]['aws_rds_cluster']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

if __name__ == '__main__':
    unittest.main()



from aws_cdk import core
from aws_cdk import aws_cloudtrail as cloudtrail

class MyCloudTrailTrailStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define CloudTrail Trail with a specific KMS Key ID
        cloudtrail.CfnTrail(
            self, 'MyCloudTrail',
            kms_key_id='arn:aws:kms:REGION:ACCOUNT_ID:key/KMS_KEY_ID',  # Replace with your KMS Key ID ARN
            # Other properties for your CloudTrail Trail
        )

app = core.App()
MyCloudTrailTrailStack(app, "MyCloudTrailTrailStack")
app.synth()



from aws_cdk import core
from aws_cdk import aws_dms as dms

class MyDMSReplicationInstanceStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define DMS Replication Instance with PubliclyAccessible set to False
        dms.ReplicationInstance(
            self, 'MyDMSReplicationInstance',
            replication_instance_identifier='MyReplicationInstance',
            allocated_storage=100,
            engine_version='3.4.3',
            publicly_accessible=True  # Set PubliclyAccessible to False
            # Add other properties as needed for your replication instance
        )

app = core.App()
MyDMSReplicationInstanceStack(app, "MyDMSReplicationInstanceStack")
app.synth()



import unittest

from checkov.common.models.enums import CheckResult
from checkov.terraform.checks.resource.aws.ECSTaskDefinitionEFSVolumeEncryption import check
import hcl2

class TestECSTaskDefinitionEFSVolumeEncryption(unittest.TestCase):

    def test_success_no_volume(self):
        hcl_res = hcl2.loads("""
resource "aws_ecs_task_definition" "test" {
  family                = "service"
  container_definitions = file("task-definitions/service.json")

  volume {
    name      = "service-storage"
    host_path = "/ecs/service-storage"
  }

  placement_constraints {
    type       = "memberOf"
    expression = "attribute:ecs.availability-zone in [us-west-2a, us-west-2b]"
  }
}
        """)
        resource_conf = hcl_res['resource'][0]['aws_ecs_task_definition']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
resource "aws_ecs_task_definition" "test" {
  family                = "service"
  container_definitions = file("task-definitions/service.json")

  volume {
    name = "service-storage"

    efs_volume_configuration {
      file_system_id          = aws_efs_file_system.fs.id
      root_directory          = "/opt/data"
      transit_encryption      = "ENABLED"
      transit_encryption_port = 2999
      authorization_config {
        access_point_id = aws_efs_access_point.test.id
        iam             = "ENABLED"
      }
    }
  }
}
        """)
        resource_conf = hcl_res['resource'][0]['aws_ecs_task_definition']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)
        
    def test_failure(self):
        hcl_res = hcl2.loads("""
resource "aws_ecs_task_definition" "test" {
  family                = "service"
  container_definitions = file("task-definitions/service.json")

  volume {
    name = "service-storage"

    efs_volume_configuration {
      file_system_id          = aws_efs_file_system.fs.id
      root_directory          = "/opt/data"
      transit_encryption_port = 2999
      authorization_config {
        access_point_id = aws_efs_access_point.test.id
        iam             = "ENABLED"
      }
    }
  }
}
        """)
        resource_conf = hcl_res['resource'][0]['aws_ecs_task_definition']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)


    def test_failure_explicit(self):
        hcl_res = hcl2.loads("""
resource "aws_ecs_task_definition" "test" {
  family                = "service"
  container_definitions = file("task-definitions/service.json")

  volume {
    name = "service-storage"

    efs_volume_configuration {
      file_system_id          = aws_efs_file_system.fs.id
      root_directory          = "/opt/data"
      transit_encryption      = "DISABLED"
      transit_encryption_port = 2999
      authorization_config {
        access_point_id = aws_efs_access_point.test.id
        iam             = "ENABLED"
      }
    }
  }
}
        """)
        resource_conf = hcl_res['resource'][0]['aws_ecs_task_definition']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

if __name__ == '__main__':
    unittest.main()



import json
from datetime import datetime
from typing import Dict, Any

import pytest
from lark import Tree

from checkov.common.util.json_utils import CustomJSONEncoder


@pytest.mark.parametrize(
    "input_dict",
    [
        pytest.param({"key": {"v", "val", "value"}}, id="set"),
        pytest.param({"key": datetime.now()}, id="date"),
        pytest.param({"key": Tree("data", ["child_1", "child_2"])}, id="lark_tree"),
        pytest.param({"key": lambda x: x}, id="function"),
        pytest.param({("key", "key2"): "value"}, id="tuple_key"),
    ],
)
def test_custom_json_encoder(input_dict: Dict[str, Any]):
    # when
    result = json.dumps(input_dict, cls=CustomJSONEncoder)

    # then
    # this assertion should never fail, but json.dumps() could
    assert isinstance(result, str)



import unittest

from checkov.main import Checkov


class ConfigException(Exception):
    pass


# override parser.error, which prints the error and exits
def parser_error(message: str):
    raise ConfigException(message)


class TestCLIArgs(unittest.TestCase):
    def test_normalize_frameworks(self):
        argv = []
        ckv = Checkov(argv=argv)
        self.assertEqual(ckv.config.framework, ['all'])
        self.assertEqual(ckv.config.skip_framework, [])

        argv = ['--framework', 'terraform']
        ckv = Checkov(argv=argv)
        self.assertEqual(ckv.config.framework, ['terraform'])

        argv = ['--framework', 'terraform,arm']
        ckv = Checkov(argv=argv)
        self.assertEqual(set(ckv.config.framework), {'terraform', 'arm'})

        argv = ['--framework', 'terraform', 'arm']
        ckv = Checkov(argv=argv)
        self.assertEqual(set(ckv.config.framework), {'terraform', 'arm'})

        argv = ['--framework', 'terraform', '--framework', 'arm']
        ckv = Checkov(argv=argv)
        self.assertEqual(set(ckv.config.framework), {'terraform', 'arm'})

        argv = ['--framework', 'terraform,bicep', '--framework', 'arm']
        ckv = Checkov(argv=argv)
        self.assertEqual(set(ckv.config.framework), {'terraform', 'arm', 'bicep'})

        argv = ['--framework', 'terraform,bicep', '--framework', 'arm,all']
        ckv = Checkov(argv=argv)
        self.assertEqual(ckv.config.framework, ['all'])

        argv = ['--framework', 'terraform,bicep', '--framework', 'arm,invalid']
        ckv = Checkov(argv=[])  # first instantiate a valid one
        # now repeat some of the logic of the constructor, overriding values
        ckv.config = ckv.parser.parse_args(argv)
        ckv.parser.error = parser_error
        with self.assertRaises(ConfigException):
            ckv.normalize_config()

        # all is specified, so we do not expect an exception
        argv = ['--framework', 'terraform,bicep', '--framework', 'arm,invalid,all']
        ckv = Checkov(argv=argv)
        self.assertEqual(ckv.config.framework, ['all'])

    def test_normalize_skip_frameworks(self):
        argv = ['--skip-framework', 'terraform']
        ckv = Checkov(argv=argv)
        self.assertEqual(ckv.config.skip_framework, ['terraform'])

        argv = ['--skip-framework', 'terraform,arm']
        ckv = Checkov(argv=argv)
        self.assertEqual(set(ckv.config.skip_framework), {'terraform', 'arm'})

        argv = ['--skip-framework', 'terraform', 'arm']
        ckv = Checkov(argv=argv)
        self.assertEqual(set(ckv.config.skip_framework), {'terraform', 'arm'})

        argv = ['--skip-framework', 'terraform', '--skip-framework', 'arm']
        ckv = Checkov(argv=argv)
        self.assertEqual(set(ckv.config.skip_framework), {'terraform', 'arm'})

        argv = ['--skip-framework', 'terraform,bicep', '--skip-framework', 'arm']
        ckv = Checkov(argv=argv)
        self.assertEqual(set(ckv.config.skip_framework), {'terraform', 'arm', 'bicep'})

        # all is not allowed
        argv = ['--skip-framework', 'terraform,bicep', '--skip-framework', 'arm,all']
        ckv = Checkov(argv=[])
        ckv.config = ckv.parser.parse_args(argv)
        ckv.parser.error = parser_error
        with self.assertRaises(ConfigException):
            ckv.normalize_config()

        argv = ['--skip-framework', 'terraform,bicep', '--skip-framework', 'arm,invalid']
        ckv = Checkov(argv=[])
        ckv.config = ckv.parser.parse_args(argv)
        ckv.parser.error = parser_error
        with self.assertRaises(ConfigException):
            ckv.normalize_config()

    def test_combine_framework_and_skip(self):
        argv = ['--framework', 'terraform', '--skip-framework', 'arm']
        ckv = Checkov(argv=argv)
        self.assertEqual(ckv.config.framework, ['terraform'])
        self.assertEqual(ckv.config.skip_framework, ['arm'])

        # duplicate values not allowed
        argv = ['--framework', 'arm', '--skip-framework', 'arm']
        ckv = Checkov(argv=[])
        ckv.config = ckv.parser.parse_args(argv)
        ckv.parser.error = parser_error
        with self.assertRaises(ConfigException):
            ckv.normalize_config()

        # but it works with all
        argv = ['--framework', 'arm,all', '--skip-framework', 'arm']
        ckv = Checkov(argv=argv)
        self.assertEqual(ckv.config.framework, ['all'])
        self.assertEqual(ckv.config.skip_framework, ['arm'])

    def test_custom_tool_name(self):
        # try using a non-standard tool name
        argv = ["--custom-tool-name", "non_standard_name"]
        ckv = Checkov(argv=argv)
        self.assertEqual(ckv.config.custom_tool_name, 'non_standard_name')

        # what about a standard tool name?
        argv = []
        ckv = Checkov(argv=argv)
        self.assertEqual(ckv.config.custom_tool_name, 'Checkov')


if __name__ == '__main__':
    unittest.main()



from aws_cdk import core
from aws_cdk import aws_cloudfront as cloudfront

class MyCloudFrontDistributionStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        distribution = cloudfront.CfnDistribution(
            self, 'MyCloudFrontDistribution',
            distribution_config=cloudfront.CfnDistribution.DistributionConfigProperty(
                viewer_certificate=cloudfront.CfnDistribution.ViewerCertificateProperty(
                    cloudfront_default_certificate=False,
                    minimum_protocol_version='TLSv1.1'  # Define the minimum supported TLS version
                ),
                # Other distribution configuration properties
            )
        )

app = core.App()
MyCloudFrontDistributionStack(app, "MyCloudFrontDistributionStack")
app.synth()

class MyCloudFrontDistributionStack2(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        distribution = cloudfront.CfnDistribution(
            self, 'MyCloudFrontDistribution',
            distribution_config=cloudfront.CfnDistribution.DistributionConfigProperty(
                viewer_certificate=cloudfront.CfnDistribution.ViewerCertificateProperty(
                    cloudfront_default_certificate=False,
                    minimum_protocol_version='TLSv1.0'  # Define the minimum supported TLS version
                ),
                # Other distribution configuration properties
            )
        )

app = core.App()
MyCloudFrontDistributionStack2(app, "MyCloudFrontDistributionStack2")
app.synth()



import unittest
from pathlib import Path

from checkov.secrets.parsers.yaml.multiline_parser import yml_multiline_parser


class TestMultilineParserYml(unittest.TestCase):
	def setUp(self) -> None:
		self.yml_parser = yml_multiline_parser

	def test_are_lines_same_indentation_yml(self):
		test_file_path = Path(__file__).parent / "resources/cfn/secret.yml"

		result = {0: True, 1: False, 2: False, 3: False, 4: False, 5: True, 6: False, 7: True,
				  8: True, 9: True, 10: True, 11: False, 12: False, 13: False, 14: False, 15: False,
				  16: True, 17: False, 18: False, 19: True, 20: True, 21: True}
		with open(file=test_file_path) as f:
			lines = f.readlines()
			# assert len(result) == len(lines)-1
			for i in range(len(lines) - 1):
				result[i] = self.yml_parser.lines_same_indentation(lines[i], lines[i + 1])

		assert result

	def test_line_is_comment_yml(self):
		examples = [
			(True, "# comment"),
			(True, "     # also comment"),
			(True, "// nice comment here"),
			(True, "//and nice comment here2"),
			(True, "      // commenting with checkov and having fun"),
			(False, "var: a  //this is not a comment"),
			(False, "var: not a comment # comment"),
			(False, "  - var: a"),
			(False, "var: "),
		]

		for ans, line in examples:
			assert ans == self.yml_parser.is_line_comment(line)


import os
import unittest
from pathlib import Path
from unittest import mock

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.bridgecrew.severities import Severities, BcSeverities
from checkov.runner_filter import RunnerFilter
from checkov.kustomize.runner import Runner, _get_blocked_remote_refs, _get_kustomization_remote_refs
from tests.kustomize.utils import kustomize_exists


def _setup_test_under_example():
    current_dir = os.path.dirname(os.path.realpath(__file__))
    scan_dir_path = os.path.join(current_dir, "runner", "resources", "example")
    # this is the relative path to the directory to scan (what would actually get passed to the -d arg)
    dir_rel_path = os.path.relpath(scan_dir_path).replace('\\', '/')
    runner = Runner()
    runner.templateRendererCommand = "kustomize"
    runner.templateRendererCommandOptions = "build"
    checks_allowlist = ['CKV_K8S_37']
    return checks_allowlist, dir_rel_path, runner


class TestRunnerValid(unittest.TestCase):
    @unittest.skipIf(os.name == "nt" or not kustomize_exists(), "kustomize not installed or Windows OS")
    def test_runner_honors_enforcement_rules(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        scan_dir_path = os.path.join(current_dir, "runner", "resources", "example")

        # this is the relative path to the directory to scan (what would actually get passed to the -d arg)
        dir_rel_path = os.path.relpath(scan_dir_path).replace('\\', '/')

        runner = Runner()
        runner.templateRendererCommand = "kustomize"
        runner.templateRendererCommandOptions = "build"
        filter = RunnerFilter(framework=['kustomize'], use_enforcement_rules=True)
        # this is not quite a true test, because the checks don't have severities. However, this shows that the check registry
        # passes the report type properly to RunnerFilter.should_run_check, and we have tests for that method
        filter.enforcement_rule_configs = {CheckType.KUSTOMIZE: Severities[BcSeverities.OFF]}
        report = runner.run(root_folder=dir_rel_path, external_checks_dir=None,
                            runner_filter=filter)

        self.assertEqual(len(report.failed_checks), 0)
        self.assertEqual(len(report.passed_checks), 0)
        self.assertEqual(len(report.skipped_checks), 0)
        self.assertEqual(len(report.parsing_errors), 0)

    @unittest.skipIf(os.name == "nt" or not kustomize_exists(), "kustomize not installed or Windows OS")
    def test_record_relative_path_with_relative_dir(self):
        # test whether the record's repo_file_path is correct, relative to the CWD (with a / at the start).

        # this is just constructing the scan dir as normal
        checks_allowlist, dir_rel_path, runner = _setup_test_under_example()
        report = runner.run(root_folder=dir_rel_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['kustomize'], checks=checks_allowlist))

        all_checks = report.failed_checks + report.passed_checks
        self.assertGreater(len(all_checks), 0)  # ensure that the assertions below are going to do something
        for record in all_checks:
            self.assertIn(record.file_path, record.file_abs_path)
            self.assertEqual(record.repo_file_path, f'/{dir_rel_path}{record.file_path}')
            assert record.file_path.startswith(('/base', '/overlays'))

    @unittest.skipIf(os.name == "nt" or not kustomize_exists(), "kustomize not installed or Windows OS")
    def test_record_relative_path_with_relative_dir_with_origin_annotations(self):
        # test whether the record's repo_file_path is correct, relative to the CWD (with a / at the start).

        # this is just constructing the scan dir as normal
        with mock.patch.dict(os.environ, {"CHECKOV_ALLOW_KUSTOMIZE_FILE_EDITS": "True"}):
            checks_allowlist, dir_rel_path, runner = _setup_test_under_example()
            report = runner.run(root_folder=dir_rel_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['kustomize'], checks=checks_allowlist))

        all_checks = report.failed_checks + report.passed_checks
        self.assertGreater(len(all_checks), 0)  # ensure that the assertions below are going to do something
        for record in all_checks:
            self.assertIn(record.file_path, record.file_abs_path)
            self.assertEqual(record.repo_file_path, f'/{dir_rel_path}{record.file_path}')
            assert record.file_path.startswith(('/base', '/overlays'))
            assert record.caller_file_path == '/base/deployment.yaml' or record.caller_file_path == '/deployment.yaml'
            assert record.caller_file_line_range == (2, 24)

    @unittest.skipIf(os.name == "nt" or not kustomize_exists(), "kustomize not installed or Windows OS")
    def test_record_relative_path_with_direct_oberlay(self):
        # test whether the record's repo_file_path is correct, relative to the CWD (with a / at the start).

        # this is just constructing the scan dir as normal
        scan_dir_path = Path(__file__).parent / "runner/resources/example/overlays/dev"


        # this is the relative path to the directory to scan (what would actually get passed to the -d arg)
        dir_rel_path = os.path.relpath(scan_dir_path).replace('\\', '/')

        runner = Runner()
        runner.templateRendererCommand = "kustomize"
        runner.templateRendererCommandOptions = "build"
        checks_allowlist = ['CKV_K8S_37']
        report = runner.run(root_folder=dir_rel_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['kustomize'], checks=checks_allowlist))

        all_checks = report.failed_checks + report.passed_checks
        self.assertGreater(len(all_checks), 0)  # ensure that the assertions below are going to do something
        for record in all_checks:
            self.assertNotEqual(record.file_path, record.file_abs_path)
            self.assertIn(record.file_path, record.file_abs_path)
            self.assertEqual(record.repo_file_path, f'/{dir_rel_path}{record.file_path}')

    @unittest.skipIf(os.name == "nt" or not kustomize_exists(), "kustomize not installed or Windows OS")
    def test_record_relative_path_with_direct_prod2_oberlay(self):
        # test whether the record's repo_file_path is correct, relative to the CWD (with a / at the start).

        # this is just constructing the scan dir as normal
        scan_dir_path = Path(__file__).parent / "runner/resources/example/overlays/prod-2"


        # this is the relative path to the directory to scan (what would actually get passed to the -d arg)
        dir_rel_path = os.path.relpath(scan_dir_path).replace('\\', '/')

        runner = Runner()
        runner.templateRendererCommand = "kustomize"
        runner.templateRendererCommandOptions = "build"
        checks_allowlist = ['CKV_K8S_37']
        report = runner.run(root_folder=dir_rel_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['kustomize'], checks=checks_allowlist))

        all_checks = report.failed_checks + report.passed_checks
        self.assertGreater(len(all_checks), 0)  # ensure that the assertions below are going to do something
        for record in all_checks:
            self.assertNotEqual(record.file_path, record.file_abs_path)
            self.assertIn(record.file_path, record.file_abs_path)
            self.assertEqual(record.repo_file_path, f'/{dir_rel_path}{record.file_path}')

    
    @unittest.skipIf(os.name == "nt" or not kustomize_exists(), "kustomize not installed or Windows OS")
    def test_no_file_type_exists(self):
        # test whether the record's repo_file_path is correct, relative to the CWD (with a / at the start).

        # this is just constructing the scan dir as normal
        scan_dir_path = Path(__file__).parent / "runner/resources/example/no_type"


        # this is the relative path to the directory to scan (what would actually get passed to the -d arg)
        dir_rel_path = os.path.relpath(scan_dir_path).replace('\\', '/')

        runner = Runner()
        runner.templateRendererCommand = "kustomize"
        runner.templateRendererCommandOptions = "build"
        report = runner.run(root_folder=dir_rel_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['kustomize']))

        all_checks = report.failed_checks + report.passed_checks
        self.assertEqual(len(all_checks), 0)  # we should no get any results

    @unittest.skipIf(os.name == "nt" or not kustomize_exists(), "kustomize not installed or Windows OS")
    def test_get_binary_output_from_directory_equals_to_get_binary_result(self):
        scan_dir_path = Path(__file__).parent / "runner/resources/example/no_type"
        dir_rel_path = os.path.relpath(scan_dir_path).replace('\\', '/')
        runner = Runner()
        runner.templateRendererCommand = "kustomize"
        runner.templateRendererCommandOptions = "build"

        # Runs the runner fully just to build `runner.kustomizeProcessedFolderAndMeta`
        _ = runner.run(root_folder=dir_rel_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['kustomize']))
        regular_result = runner.get_binary_output(str(scan_dir_path), runner.kustomizeProcessedFolderAndMeta,
                                                  runner.templateRendererCommand)
        result_from_directory = runner.get_binary_output_from_directory(str(scan_dir_path),
                                                                        runner.templateRendererCommand)
        assert regular_result == result_from_directory



class TestRemoteBaseAllowlist(unittest.TestCase):
    """Tests for kustomize remote base allowlist.

    Default behaviour: kustomize runs normally (original behaviour preserved).
    Blocking is opt-in via CHECKOV_KUSTOMIZE_ALLOWED_REMOTE_PREFIXES — an allowlist of
    trusted URL prefixes. Remote refs NOT in the allowlist are blocked.
    """

    _REMOTE_BASE_DIR = str(Path(__file__).parent / "runner/resources/example_remote_base")

    def _make_runner(self) -> Runner:
        runner = Runner()
        runner.templateRendererCommand = "kustomize"
        return runner

    # ------------------------------------------------------------------
    # _get_kustomization_remote_refs() unit tests
    # ------------------------------------------------------------------

    def test_get_kustomization_remote_refs_detects_http_url(self):
        """_get_kustomization_remote_refs() must return http:// entries."""
        refs = _get_kustomization_remote_refs(self._REMOTE_BASE_DIR)
        self.assertIn("http://192.0.2.1/example-base.yaml", refs)

    def test_get_kustomization_remote_refs_detects_git_url(self):
        """_get_kustomization_remote_refs() must return git:: entries."""
        refs = _get_kustomization_remote_refs(self._REMOTE_BASE_DIR)
        self.assertTrue(any(r.startswith("git::") for r in refs))

    def test_get_kustomization_remote_refs_returns_empty_for_local_only(self):
        """_get_kustomization_remote_refs() must return [] for local-only kustomizations."""
        local_dir = str(Path(__file__).parent / "runner/resources/example/base")
        self.assertEqual(_get_kustomization_remote_refs(local_dir), [])

    def test_get_kustomization_remote_refs_returns_empty_for_missing_dir(self):
        """_get_kustomization_remote_refs() must not raise for a missing directory."""
        self.assertEqual(_get_kustomization_remote_refs("/nonexistent/path"), [])

    # ------------------------------------------------------------------
    # _get_blocked_remote_refs() unit tests (allowlist logic)
    # ------------------------------------------------------------------

    def test_get_blocked_remote_refs_returns_empty_when_no_env_var(self):
        """Default: no env var → _get_blocked_remote_refs() returns [] (allow all)."""
        with mock.patch.dict(os.environ, {}, clear=False):
            os.environ.pop("CHECKOV_KUSTOMIZE_ALLOWED_REMOTE_PREFIXES", None)
            refs = _get_blocked_remote_refs(self._REMOTE_BASE_DIR)
        self.assertEqual(refs, [])

    def test_get_blocked_remote_refs_returns_blocked_when_not_in_allowlist(self):
        """Remote refs NOT in the allowlist must be returned (they will be blocked)."""
        env = {"CHECKOV_KUSTOMIZE_ALLOWED_REMOTE_PREFIXES": "https://github.com/trusted-org/"}
        with mock.patch.dict(os.environ, env):
            refs = _get_blocked_remote_refs(self._REMOTE_BASE_DIR)
        self.assertIn("http://192.0.2.1/example-base.yaml", refs)

    def test_get_blocked_remote_refs_returns_empty_when_all_in_allowlist(self):
        """When all remote refs are in the allowlist, nothing is blocked."""
        env = {"CHECKOV_KUSTOMIZE_ALLOWED_REMOTE_PREFIXES": "http://192.0.2.1/,git::https://example.com/"}
        with mock.patch.dict(os.environ, env):
            refs = _get_blocked_remote_refs(self._REMOTE_BASE_DIR)
        self.assertEqual(refs, [])

    def test_get_blocked_remote_refs_returns_empty_for_local_only(self):
        """Local-only kustomizations are never blocked."""
        local_dir = str(Path(__file__).parent / "runner/resources/example/base")
        env = {"CHECKOV_KUSTOMIZE_ALLOWED_REMOTE_PREFIXES": "https://github.com/trusted-org/"}
        with mock.patch.dict(os.environ, env):
            refs = _get_blocked_remote_refs(local_dir)
        self.assertEqual(refs, [])

    # ------------------------------------------------------------------
    # _get_kubectl_output() integration tests
    # ------------------------------------------------------------------

    def test_get_kubectl_output_allows_remote_refs_by_default(self):
        """Default (no env var): kustomize build runs even with remote refs."""
        runner = self._make_runner()
        fake_output = b"apiVersion: v1\nkind: ConfigMap\n"
        mock_proc = mock.MagicMock()
        mock_proc.communicate.return_value = (fake_output, b"")
        with mock.patch.dict(os.environ, {}, clear=False):
            os.environ.pop("CHECKOV_KUSTOMIZE_ALLOWED_REMOTE_PREFIXES", None)
            with mock.patch("checkov.kustomize.runner.subprocess.Popen", return_value=mock_proc) as mock_popen:
                result = runner._get_kubectl_output(self._REMOTE_BASE_DIR, "kustomize", "base")
        mock_popen.assert_called_once()
        self.assertEqual(result, fake_output)

    def test_get_kubectl_output_blocks_when_not_in_allowlist(self):
        """When CHECKOV_KUSTOMIZE_ALLOWED_REMOTE_PREFIXES is set and a ref is not in it, build is skipped."""
        runner = self._make_runner()
        env = {"CHECKOV_KUSTOMIZE_ALLOWED_REMOTE_PREFIXES": "https://github.com/trusted-org/"}
        with mock.patch.dict(os.environ, env):
            with mock.patch("checkov.kustomize.runner.subprocess.Popen") as mock_popen:
                result = runner._get_kubectl_output(self._REMOTE_BASE_DIR, "kustomize", "base")
        self.assertIsNone(result)
        mock_popen.assert_not_called()

    def test_get_kubectl_output_allows_when_all_in_allowlist(self):
        """When all remote refs are in the allowlist, build proceeds normally."""
        runner = self._make_runner()
        fake_output = b"apiVersion: v1\nkind: ConfigMap\n"
        mock_proc = mock.MagicMock()
        mock_proc.communicate.return_value = (fake_output, b"")
        env = {"CHECKOV_KUSTOMIZE_ALLOWED_REMOTE_PREFIXES": "http://192.0.2.1/,git::https://example.com/"}
        with mock.patch.dict(os.environ, env):
            with mock.patch("checkov.kustomize.runner.subprocess.Popen", return_value=mock_proc) as mock_popen:
                result = runner._get_kubectl_output(self._REMOTE_BASE_DIR, "kustomize", "base")
        mock_popen.assert_called_once()
        self.assertEqual(result, fake_output)

    def test_get_kubectl_output_allows_local_refs_without_env(self):
        """Local-only kustomizations always run regardless of env var."""
        runner = self._make_runner()
        local_dir = str(Path(__file__).parent / "runner/resources/example/base")
        fake_output = b"apiVersion: v1\nkind: ConfigMap\n"
        mock_proc = mock.MagicMock()
        mock_proc.communicate.return_value = (fake_output, b"")
        with mock.patch.dict(os.environ, {}, clear=False):
            os.environ.pop("CHECKOV_KUSTOMIZE_ALLOWED_REMOTE_PREFIXES", None)
            with mock.patch("checkov.kustomize.runner.subprocess.Popen", return_value=mock_proc) as mock_popen:
                result = runner._get_kubectl_output(local_dir, "kustomize", "base")
        mock_popen.assert_called_once()
        self.assertEqual(result, fake_output)


if __name__ == '__main__':
    unittest.main()



import unittest
from pathlib import Path

from checkov.runner_filter import RunnerFilter
from checkov.terraform.checks.resource.aws.EKSControlPlaneLogging import check
from checkov.terraform.runner import Runner
from checkov.common.models.enums import CheckResult


class TestEKSControlPlaneLogging(unittest.TestCase):
    def test_failure(self):
        resource_conf = {'name': ['testcluster'], 'enabled_cluster_log_types': [['api', 'audit']]}

        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_empty(self):
        resource_conf = {'name': ['testcluster']}

        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        resource_conf = {'name': ['testcluster'], 'enabled_cluster_log_types': [['api', 'audit', 'authenticator', 'controllerManager', 'scheduler']]}

        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_failure_not_enabled(self):
        resource_conf = {'name': ['testcluster'], 'enabled_cluster_log_types': []}

        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_file(self):
        # given
        test_files_dir = Path(__file__).parent / "example_EKSControlPlaneLogging"

        # when
        report = Runner().run(root_folder=str(test_files_dir), runner_filter=RunnerFilter(checks=[check.id]))

        # then
        summary = report.get_summary()

        passing_resources = {
            "aws_eks_cluster.fully_enabled",
            "aws_eks_cluster.fully_enabled_with_dynamic_block"
        }
        failing_resources = {
            "aws_eks_cluster.partially_enabled",
            "aws_eks_cluster.not_configured"
        }

        passed_check_resources = {c.resource for c in report.passed_checks}
        failed_check_resources = {c.resource for c in report.failed_checks}

        self.assertEqual(summary["passed"], 2)
        self.assertEqual(summary["failed"], 2)
        self.assertEqual(summary["skipped"], 0)
        self.assertEqual(summary["parsing_errors"], 0)

        self.assertEqual(passing_resources, passed_check_resources)
        self.assertEqual(failing_resources, failed_check_resources)



if __name__ == '__main__':
    unittest.main()



from aws_cdk import core
from aws_cdk import aws_lambda as _lambda
from aws_cdk import aws_sam as sam

class MyLambdaFunctionStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define Lambda Function
        my_lambda = _lambda.Function(
            self, 'MyLambdaFunction',
            runtime=_lambda.Runtime.PYTHON_3_8,
            handler='index.handler',
            code=_lambda.Code.from_asset('lambda'),  # Replace 'lambda' with your function code directory
            environment={
                'MY_VARIABLE': {'a':'b'}
            }
        )

app = core.App()
MyLambdaFunctionStack(app, "MyLambdaFunctionStack")
app.synth()


class MyServerlessFunctionStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define Serverless Lambda Function
        my_lambda = sam.CfnFunction(
            self, 'MyServerlessFunction',
            code_uri='lambda/',  # Replace 'lambda/' with your function code directory
            handler='index.handler',
            runtime='python3.9',
            environment={
                'MY_VARIABLE': {'a':'b'}
            }
            # Other properties for your Serverless Lambda Function
        )

app = core.App()
MyServerlessFunctionStack(app, "MyServerlessFunctionStack")
app.synth()



from __future__ import annotations

import logging
from pathlib import Path
from typing import TYPE_CHECKING

import pytest

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.runners.runner_registry import RunnerRegistry
from checkov.main import DEFAULT_RUNNERS
from checkov.runner_filter import RunnerFilter

if TYPE_CHECKING:
    from _pytest.logging import LogCaptureFixture


TESTS_DIR = Path(__file__).parent.parent / "tests"


def test_all_frameworks_are_tested() -> None:
    # given
    checkov_runners = {value for attr, value in CheckType.__dict__.items() if not attr.startswith("__")}

    # remove frameworks, which are not applicable
    checkov_runners.difference_update(
        {
            CheckType.BITBUCKET_CONFIGURATION,
            CheckType.GITHUB_CONFIGURATION,
            CheckType.GITLAB_CONFIGURATION,
            CheckType.JSON,
            CheckType.SCA_IMAGE,
            CheckType.SCA_PACKAGE,
            CheckType.YAML,
        }
    )

    assert checkov_runners == {
        CheckType.ANSIBLE,
        CheckType.ARGO_WORKFLOWS,
        CheckType.ARM,
        CheckType.AZURE_PIPELINES,
        CheckType.BICEP,
        CheckType.BITBUCKET_PIPELINES,
        CheckType.CDK,
        CheckType.CIRCLECI_PIPELINES,
        CheckType.CLOUDFORMATION,
        CheckType.DOCKERFILE,
        CheckType.GITHUB_ACTIONS,
        CheckType.GITLAB_CI,
        CheckType.HELM,
        CheckType.KUBERNETES,
        CheckType.KUSTOMIZE,
        CheckType.OPENAPI,
        CheckType.SAST,
        CheckType.SAST_JAVA,
        CheckType.SAST_PYTHON,
        CheckType.SAST_JAVASCRIPT,
        CheckType.SAST_TYPESCRIPT,
        CheckType.SAST_GOLANG,
        CheckType.SECRETS,
        CheckType.SERVERLESS,
        CheckType.TERRAFORM,
        CheckType.TERRAFORM_JSON,
        CheckType.TERRAFORM_PLAN,
        CheckType.POLICY_3D
    }, "Don't forget to add a test case for the new runner here"


def test_ansible_framework(caplog: LogCaptureFixture) -> None:
    run_framework_test(caplog=caplog, framework=CheckType.ANSIBLE)


def test_argo_workflows_framework(caplog: LogCaptureFixture) -> None:
    run_framework_test(caplog=caplog, framework=CheckType.ARGO_WORKFLOWS)


def test_arm_framework(caplog: LogCaptureFixture) -> None:
    excluded_paths = ["arm/parser/examples/json/with_comments.json$"]

    run_framework_test(caplog=caplog, framework=CheckType.ARM, excluded_paths=excluded_paths)


def test_azure_pipelines_framework(caplog: LogCaptureFixture) -> None:
    run_framework_test(caplog=caplog, framework=CheckType.AZURE_PIPELINES)


def test_bicep_framework(caplog: LogCaptureFixture) -> None:
    excluded_paths = ["bicep/examples/malformed.bicep$"]

    run_framework_test(caplog=caplog, framework=CheckType.BICEP, excluded_paths=excluded_paths)


def test_bitbucket_pipelines_framework(caplog: LogCaptureFixture) -> None:
    run_framework_test(caplog=caplog, framework=CheckType.BITBUCKET_PIPELINES)


@pytest.mark.xfail(reason="locally it works, but in CI no results")
def test_cdk_framework(caplog: LogCaptureFixture) -> None:
    run_framework_test(caplog=caplog, framework=CheckType.CDK)


def test_circleci_pipelines_framework(caplog: LogCaptureFixture) -> None:
    run_framework_test(caplog=caplog, framework=CheckType.CIRCLECI_PIPELINES)


def test_cloudformation_framework(caplog: LogCaptureFixture) -> None:
    excluded_paths = [
        "cloudformation/parser/cfn_bad_name.yaml$",
        "cloudformation/parser/cfn_with_ref_bad.yaml$",
        "cloudformation/parser/success_triple_quotes_string.json$",
        "cloudformation/runner/resources/invalid.json$",
        "cloudformation/runner/resources/invalid.yaml$",
        "cloudformation/runner/resources/invalid_properties.json$",
        "cloudformation/runner/resources/invalid_properties.yaml$",
    ]

    run_framework_test(caplog=caplog, framework=CheckType.CLOUDFORMATION, excluded_paths=excluded_paths)


def test_dockerfile_framework(caplog: LogCaptureFixture) -> None:
    run_framework_test(caplog=caplog, framework=CheckType.DOCKERFILE)


def test_github_actions_framework(caplog: LogCaptureFixture) -> None:
    run_framework_test(caplog=caplog, framework=CheckType.GITHUB_ACTIONS)


def test_gitlab_ci_framework(caplog: LogCaptureFixture) -> None:
    run_framework_test(caplog=caplog, framework=CheckType.GITLAB_CI)


def test_helm_framework(caplog: LogCaptureFixture) -> None:
    run_framework_test(caplog=caplog, framework=CheckType.HELM)


def test_kubernetes_framework(caplog: LogCaptureFixture) -> None:
    run_framework_test(caplog=caplog, framework=CheckType.KUBERNETES)


@pytest.mark.skip(reason="kustomize needs a context to do a proper scan, which is hard to set here")
def test_kustomize_framework(caplog: LogCaptureFixture) -> None:
    run_framework_test(caplog=caplog, framework=CheckType.KUSTOMIZE)


def test_openapi_framework(caplog: LogCaptureFixture) -> None:
    run_framework_test(caplog=caplog, framework=CheckType.OPENAPI)


def test_secrets_framework(caplog: LogCaptureFixture) -> None:
    run_framework_test(caplog=caplog, framework=CheckType.SECRETS)


def test_serverless_framework(caplog: LogCaptureFixture) -> None:
    run_framework_test(caplog=caplog, framework=CheckType.SERVERLESS)


def test_terraform_framework(caplog: LogCaptureFixture) -> None:
    excluded_paths = [
        "terraform/runner/resources/example/invalid.tf$",
        "terraform/runner/resources/invalid_terraform_syntax/bad_tf_1.tf$",
        "terraform/runner/resources/invalid_terraform_syntax/bad_tf_2.tf$",
        "terraform/runner/resources/unbalanced_eval_brackets/main.tf$",
        "terraform/runner/resources/hcl_timeout/main.tf$",
        "terraform/parser/resources/hcl_timeout/main.tf$",
    ]

    run_framework_test(caplog=caplog, framework=CheckType.TERRAFORM, excluded_paths=excluded_paths)


def test_terraform_json_framework(caplog: LogCaptureFixture) -> None:
    run_framework_test(caplog=caplog, framework=CheckType.TERRAFORM_JSON)


def test_terraform_plan_framework(caplog: LogCaptureFixture) -> None:
    excluded_paths = [
        "arm/parser/examples/json/with_comments.json$",
        "cloudformation/parser/fail.json$",
        "cloudformation/parser/success_triple_quotes_string.json$",
        "cloudformation/runner/resources/invalid.json$",
    ]

    run_framework_test(caplog=caplog, framework=CheckType.TERRAFORM_PLAN, excluded_paths=excluded_paths)


def run_framework_test(caplog: LogCaptureFixture, framework: str, excluded_paths: list[str] | None = None) -> None:
    # given
    caplog.set_level(logging.ERROR)
    runner_registry = RunnerRegistry(
        "", RunnerFilter(framework=[framework], excluded_paths=excluded_paths), *DEFAULT_RUNNERS
    )

    # when
    scan_reports = runner_registry.run(root_folder=str(TESTS_DIR))

    # then
    for report in scan_reports:
        assert report.failed_checks
        assert not report.parsing_errors, f"Found parsing errors for framework '{report.check_type}'"

    assert not caplog.text, caplog.text



import unittest

import hcl2

from checkov.terraform.checks.resource.azure.PostgresSQLTreatDetectionEnabled import check
from checkov.common.models.enums import CheckResult


class TestPostgresSQLTreatDetectionEnabled(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_postgresql_server" "example" {
              name                = "example-mysqlserver"
              location            = azurerm_resource_group.example.location
              resource_group_name = azurerm_resource_group.example.name
            
              administrator_login          = "mysqladminun"
              administrator_login_password = "H@Sh1CoR3!"  # checkov:skip=CKV_SECRET_80 test secret
            
              sku_name   = "B_Gen5_2"
              storage_mb = 5120
              version    = "5.7"
            
              auto_grow_enabled                 = true
              backup_retention_days             = 7
              geo_redundant_backup_enabled      = true
              infrastructure_encryption_enabled = true
              public_network_access_enabled     = false
              ssl_enforcement_enabled           = false
              ssl_minimal_tls_version_enforced  = "TLS1_2"

              threat_detection_policy {
                  enabled = false
              }
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_postgresql_server']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)
        
    def test_failure_empty(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_postgresql_server" "example" {
              name                = "example-mysqlserver"
              location            = azurerm_resource_group.example.location
              resource_group_name = azurerm_resource_group.example.name
            
              administrator_login          = "mysqladminun"
              administrator_login_password = "H@Sh1CoR3!"
            
              sku_name   = "B_Gen5_2"
              storage_mb = 5120
              version    = "5.7"
            
              auto_grow_enabled                 = true
              backup_retention_days             = 7
              geo_redundant_backup_enabled      = true
              infrastructure_encryption_enabled = true
              public_network_access_enabled     = false
              ssl_enforcement_enabled           = false
              ssl_minimal_tls_version_enforced  = "TLS1_2"
              
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_postgresql_server']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_postgresql_server" "example" {
              name                = "example-mysqlserver"
              location            = azurerm_resource_group.example.location
              resource_group_name = azurerm_resource_group.example.name
            
              administrator_login          = "mysqladminun"
              administrator_login_password = "H@Sh1CoR3!"
            
              sku_name   = "B_Gen5_2"
              storage_mb = 5120
              version    = "5.7"
            
              auto_grow_enabled                 = true
              backup_retention_days             = 7
              geo_redundant_backup_enabled      = true
              infrastructure_encryption_enabled = true
              public_network_access_enabled     = false
              ssl_enforcement_enabled           = true
              ssl_minimal_tls_version_enforced  = "TLS1_2"
              threat_detection_policy {
                  enabled = true
              }
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_postgresql_server']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



import unittest
import hcl2

from checkov.terraform.checks.resource.aws.DMSReplicationInstancePubliclyAccessible import check
from checkov.common.models.enums import CheckResult


class TestDMSReplicationInstancePubliclyAccessible(unittest.TestCase):


    def test_failure_set_public(self):
        hcl_res = hcl2.loads("""
          resource "aws_dms_replication_instance" "public" {
            engine_version               = "3.1.4"
            multi_az                     = false
            publicly_accessible          = true
            replication_instance_class   = "dms.t2.micro"
            replication_instance_id      = "test-dms-replication-instance-tf"
          }
        """)
        resource_conf = hcl_res['resource'][0]['aws_dms_replication_instance']['public']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success_set_private(self):
        hcl_res = hcl2.loads("""
          resource "aws_dms_replication_instance" "private" {
            engine_version               = "3.1.4"
            multi_az                     = false
            publicly_accessible          = false
            replication_instance_class   = "dms.t2.micro"
            replication_instance_id      = "test-dms-replication-instance-tf"
          }
        """)
        resource_conf = hcl_res['resource'][0]['aws_dms_replication_instance']['private']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_default(self):
        hcl_res = hcl2.loads("""
          resource "aws_dms_replication_instance" "private" {
            engine_version               = "3.1.4"
            multi_az                     = false
            replication_instance_class   = "dms.t2.micro"
            replication_instance_id      = "test-dms-replication-instance-tf"
          }
        """)
        resource_conf = hcl_res['resource'][0]['aws_dms_replication_instance']['private']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

if __name__ == '__main__':
    unittest.main()



from pathlib import Path

import pytest

from checkov.common.graph.db_connectors.networkx.networkx_db_connector import NetworkxConnector
from checkov.common.graph.db_connectors.rustworkx.rustworkx_db_connector import RustworkxConnector
from checkov.runner_filter import RunnerFilter
from checkov.terraform.runner import Runner

@pytest.mark.parametrize(
    "graph_connector",
    [
        NetworkxConnector,
        RustworkxConnector
    ]
)
def test_dynamics(graph_connector):
    # given
    test_files_dir = Path(__file__).parent.parent / "resources/dynamic_lambda_function"

    # when
    report = Runner(db_connector=graph_connector()).run(
        root_folder=str(test_files_dir),
        runner_filter=RunnerFilter(
            checks=[
                "CKV_AWS_45",
                "CKV_AWS_116",
                "CKV_AWS_173",
                "CKV_AWS_272",
            ]
        ),
    )

    # then
    summary = report.get_summary()

    assert summary["passed"] == 2
    assert summary["failed"] == 2
    assert summary["skipped"] == 0
    assert summary["parsing_errors"] == 0



import json
import logging
import os
from pathlib import Path
from unittest import mock

import pytest
from aioresponses import aioresponses
from pytest_mock import MockerFixture

from checkov.common.bridgecrew.vulnerability_scanning.image_scanner import (
    ImageScanner,
    CHECKOV_SEC_IN_WEEK,
    DOCKER_IMAGE_SCAN_RESULT_FILE_NAME,
    redact_token_args,
)
from checkov.common.bridgecrew.vulnerability_scanning.integrations.docker_image_scanning import (
    docker_image_scanning_integration,
)


def get_report_url() -> str:
    base_url = "https://www.bridgecrew.cloud/api/v1/vulnerabilities"
    return f"{base_url}/results"


@pytest.mark.asyncio
async def test_report_results(mocker: MockerFixture, mock_bc_integration, docker_image_scan_result):
    # given
    bc_api_key = "abcd1234-abcd-1234-abcd-1234abcd1234"
    report_url = get_report_url()

    mocker.patch.dict(os.environ, {"BC_ROOT_DIR": "app"})

    # when
    with aioresponses() as m:
        m.post(report_url, status=200)

        result = await docker_image_scanning_integration.report_results_async(
            twistcli_scan_result=docker_image_scan_result,
            bc_platform_integration=mock_bc_integration,
            bc_api_key=bc_api_key,
            file_path=Path("app/Dockerfile"),
            file_content="FROM python:3.7-slim\n",
            docker_image_name="sha256:e70f7611f4d093d5f73026d23f7ab612aa1794abfb210ef1c549b225380d053b",
        )

    # then
    assert result == 0
    assert next(iter(m.requests.values()))[0].kwargs["json"] == {
        "dockerImageName": "sha256:e70f7611f4d093d5f73026d23f7ab612aa1794abfb210ef1c549b225380d053b",
        "dockerFilePath": "/Dockerfile",
        "dockerFileContent": "FROM python:3.7-slim\n",
        "type": "Image",
        "sourceId": "bridgecrewio/checkov",
        "branch": "master",
        "sourceType": "Github",
        "vulnerabilities": [
            {
                "cveId": "CVE-2022-23990",
                "status": "fixed in 2.2.10-2+deb11u1",
                "severity": "critical",
                "packageName": "expat",
                "packageVersion": "2.2.10-2",
                "link": "https://security-tracker.debian.org/tracker/CVE-2022-23990",
                "cvss": 9.8,
                "vector": "CVSS:3.1/AV:N/AC:L/PR:N/UI:N/S:U/C:H/I:H/A:H",
                "description": "Expat (aka libexpat) before 2.4.4 has an integer overflow in the doProlog function.",
                "riskFactors": [
                    "Attack complexity: low",
                    "Attack vector: network",
                    "Critical severity",
                    "Has fix",
                    "Recent vulnerability",
                ],
                "publishedDate": "2022-01-26T21:15:00+02:00",
            },
            {
                "cveId": "CVE-2022-23852",
                "status": "fixed in 2.2.10-2+deb11u1",
                "severity": "critical",
                "packageName": "expat",
                "packageVersion": "2.2.10-2",
                "link": "https://security-tracker.debian.org/tracker/CVE-2022-23852",
                "cvss": 9.8,
                "vector": "CVSS:3.1/AV:N/AC:L/PR:N/UI:N/S:U/C:H/I:H/A:H",
                "description": "Expat (aka libexpat) before 2.4.4 has a signed integer overflow in XML_GetBuffer, for configurations with a nonzero XML_CONTEXT_BYTES.",
                "riskFactors": [
                    "Attack complexity: low",
                    "Attack vector: network",
                    "Critical severity",
                    "Has fix",
                    "Recent vulnerability",
                ],
                "publishedDate": "2022-01-24T04:15:00+02:00",
            },
        ],
        "packages": [
            {"type": "os", "name": "mawk", "version": "1.3.4.20200120-2", "licenses": ["GPL-2"]},
            {"type": "os", "name": "gzip", "version": "1.10-4", "licenses": ["GPL-3+"]},
        ],
        "relatedResourceId": None,
        "errorLines": None
    }


@pytest.mark.asyncio
async def test_report_results_with_cicd(mocker: MockerFixture, mock_bc_integration, docker_image_scan_result):
    # given
    bc_api_key = "abcd1234-abcd-1234-abcd-1234abcd1234"
    report_url = get_report_url()
    cicd_details = {
        "runId": 123,
        "pr": "patch-1",
        "commit": "qwerty1234",
    }

    mock_bc_integration.cicd_details = cicd_details
    mocker.patch.dict(os.environ, {"BC_ROOT_DIR": "app"})

    # when
    with aioresponses() as m:
        m.post(report_url, status=200)

        result = await docker_image_scanning_integration.report_results_async(
            twistcli_scan_result=docker_image_scan_result,
            bc_platform_integration=mock_bc_integration,
            bc_api_key=bc_api_key,
            file_path=Path("app/Dockerfile"),
            file_content="FROM python:3.7-slim\n",
            docker_image_name="sha256:e70f7611f4d093d5f73026d23f7ab612aa1794abfb210ef1c549b225380d053b",
        )

    # then
    assert result == 0
    assert next(iter(m.requests.values()))[0].kwargs["json"]["cicdDetails"] == cicd_details


@pytest.mark.asyncio
async def test_report_results_fail(mocker: MockerFixture, mock_bc_integration, docker_image_scan_result):
    # given
    bc_api_key = "abcd1234-abcd-1234-abcd-1234abcd1234"
    report_url = get_report_url()

    mocker.patch.dict(os.environ, {"BC_ROOT_DIR": "app", "REQUEST_MAX_TRIES": "3", "SLEEP_BETWEEN_REQUEST_TRIES": "0.01"})

    # when
    with aioresponses() as m:
        m.post(report_url, status=403, repeat=True)

        result = await docker_image_scanning_integration.report_results_async(
            twistcli_scan_result=docker_image_scan_result,
            bc_platform_integration=mock_bc_integration,
            bc_api_key=bc_api_key,
            file_path=Path("app/Dockerfile"),
            file_content="FROM python:3.7-slim\n",
            docker_image_name="sha256:e70f7611f4d093d5f73026d23f7ab612aa1794abfb210ef1c549b225380d053b",
        )

    # then
    assert result == 1


def test_should_download_new_twistcli(tmp_path: Path):
    # given
    scanner = ImageScanner()
    twistcli_path = tmp_path / "twistcli"
    scanner.twistcli_path = twistcli_path

    # then
    assert scanner.should_download()


@mock.patch.dict(os.environ, {"CHECKOV_EXPIRATION_TIME_IN_SEC": str(CHECKOV_SEC_IN_WEEK)})
def test_not_should_download_twistcli(tmp_path: Path):
    # given
    scanner = ImageScanner()
    twistcli_path = tmp_path / "twistcli"
    twistcli_path.touch()
    scanner.twistcli_path = twistcli_path

    # then
    assert not scanner.should_download()


@mock.patch.dict(os.environ, {"CHECKOV_EXPIRATION_TIME_IN_SEC": "0"})
def test_should_download_twistcli_again(tmp_path: Path):
    # given
    scanner = ImageScanner()
    twistcli_path = tmp_path / "twistcli"
    twistcli_path.touch()
    scanner.twistcli_path = twistcli_path

    # then
    assert scanner.should_download()


def test_cleanup_twistcli_exists(tmp_path: Path):
    # given
    scanner = ImageScanner()

    # prepare local paths
    twistcli_path = tmp_path / "twistcli"
    twistcli_path.touch()
    scanner.twistcli_path = twistcli_path

    # when
    scanner.cleanup_scan()

    # then
    assert not twistcli_path.exists()


def test_cleanup_twistcli_not_exists(tmp_path: Path):
    # given
    scanner = ImageScanner()

    # prepare local paths
    twistcli_path = tmp_path / "twistcli"
    scanner.twistcli_path = twistcli_path

    # when
    scanner.cleanup_scan()

    # then
    assert not twistcli_path.exists()


def test_redact_token_args():
    args = ["twistcli", "images", "scan", "--address", "https://addr", "--token", "super-secret", "--details", "img"]
    redacted = redact_token_args(args)

    assert redacted[redacted.index("--token") + 1] == "***"
    assert "super-secret" not in redacted
    # original list must not be mutated
    assert args[args.index("--token") + 1] == "super-secret"


def test_run_image_scan_builds_list_args(
    tmp_path: Path, monkeypatch: pytest.MonkeyPatch, mocker: MockerFixture, caplog: pytest.LogCaptureFixture
):
    # given
    scanner = ImageScanner()
    scanner.twistcli_path = tmp_path / "twistcli"
    image_id = "alpine:3.19"
    api_key = "abcd1234-abcd-1234-abcd-1234abcd1234"

    monkeypatch.chdir(tmp_path)
    (tmp_path / DOCKER_IMAGE_SCAN_RESULT_FILE_NAME).write_text(json.dumps({"results": [{}]}))

    mocker.patch.object(docker_image_scanning_integration, "get_proxy_address", return_value="https://proxy.example")
    mocker.patch.object(docker_image_scanning_integration, "get_bc_api_key", return_value=api_key)
    run_mock = mocker.patch("checkov.common.bridgecrew.vulnerability_scanning.image_scanner.subprocess.run")

    # when
    with caplog.at_level(logging.DEBUG):
        scanner.run_image_scan(image_id)

    # then
    called_args = run_mock.call_args.args[0]
    assert isinstance(called_args, list)
    assert called_args.count(image_id) == 1
    assert called_args[-1] == image_id
    assert called_args[called_args.index("--output-file") + 1] == DOCKER_IMAGE_SCAN_RESULT_FILE_NAME
    assert api_key not in caplog.text
    assert "***" in caplog.text


def test_run_image_scan_rejects_invalid_image_id(tmp_path: Path, mocker: MockerFixture):
    # given
    scanner = ImageScanner()
    scanner.twistcli_path = tmp_path / "twistcli"
    run_mock = mocker.patch("checkov.common.bridgecrew.vulnerability_scanning.image_scanner.subprocess.run")

    # when / then
    with pytest.raises(ValueError):
        scanner.run_image_scan("alpine:3.19 --extra flag")

    run_mock.assert_not_called()



from __future__ import annotations

from typing import Any

from checkov.common.graph.graph_builder import CustomAttributes
from checkov.common.graph.graph_builder.local_graph import LocalGraph, _Block
from checkov.common.util.consts import LINE_FIELD_NAMES
from checkov.common.util.data_structures_utils import pickle_deepcopy
from checkov.serverless.graph_builder.graph_components.blocks import ServerlessBlock
from checkov.serverless.utils import ServerlessElements


class ServerlessLocalGraph(LocalGraph[ServerlessBlock]):
    def __init__(self, definitions: dict[str, dict[str, Any]]) -> None:
        super().__init__()
        self.vertices: list[ServerlessBlock] = []
        self.definitions = definitions
        self.vertices_by_path_and_name: dict[tuple[str, str], int] = {}

    def build_graph(self, render_variables: bool = True) -> None:
        self._create_vertices()

    def _create_vertices(self) -> None:
        for file_path, definition in self.definitions.items():
            self._create_vertex(file_path=file_path, definition=definition, element_type=ServerlessElements.FUNCTIONS)
            self._create_vertex(file_path=file_path, definition=definition, element_type=ServerlessElements.PARAMS)
            self._create_vertex(file_path=file_path, definition=definition, element_type=ServerlessElements.PROVIDER)
            self._create_vertex(file_path=file_path, definition=definition, element_type=ServerlessElements.LAYERS)
            self._create_vertex(file_path=file_path, definition=definition, element_type=ServerlessElements.CUSTOM)
            self._create_vertex(file_path=file_path, definition=definition, element_type=ServerlessElements.PACKAGE)
            self._create_vertex(file_path=file_path, definition=definition, element_type=ServerlessElements.PLUGINS)
            self._create_vertex(file_path=file_path, definition=definition, element_type=ServerlessElements.SERVICE)
            self._create_vertex(file_path=file_path, definition=definition, element_type=ServerlessElements.RESOURCES)

        for i, vertex in enumerate(self.vertices):
            self.vertices_by_block_type[vertex.block_type].append(i)
            self.vertices_by_path_and_name[(vertex.path, vertex.name)] = i

            self.in_edges[i] = []
            self.out_edges[i] = []

    def _create_vertex(self, file_path: str, definition: dict[str, Any] | None,
                       element_type: ServerlessElements) -> None:
        if not definition:
            return

        resources = definition.get(element_type)

        # resources -> Resources
        if element_type == ServerlessElements.RESOURCES and resources is None:
            resources = definition.get('Resources')

        if isinstance(resources, list) and len(resources) > 0 and \
           isinstance(resources[0], dict) and resources[0]['__file__'] != file_path:
            for r in resources:
                if isinstance(r, dict):
                    self._create_vertex(file_path, {element_type: r}, element_type)
            return

        if not resources:
            return

        elif isinstance(resources, str):
            self.vertices.append(ServerlessBlock(
                name=f'{element_type}',
                config={"value": pickle_deepcopy(resources)},
                path=file_path,
                block_type=element_type,
                attributes={"value": pickle_deepcopy(resources)},
                id=f"{file_path}:{element_type}"
            ))

        else:
            for attribute in resources:
                if isinstance(attribute, str) and attribute in LINE_FIELD_NAMES:
                    continue

                if isinstance(resources, list):
                    full_conf = {"value": pickle_deepcopy(attribute)}
                    self.vertices.append(ServerlessBlock(
                        name=f'{element_type}.{attribute}',
                        config=full_conf,
                        path=file_path,
                        block_type=element_type,
                        attributes=full_conf,
                        id=f"{file_path}:{element_type}.{attribute}"
                    ))

                else:
                    attribute_value = resources[attribute]
                    if not isinstance(attribute_value, dict):
                        full_conf = {"value": pickle_deepcopy(attribute_value)}
                    else:
                        full_conf = attribute_value

                    config = pickle_deepcopy(full_conf)

                    resource_type = element_type

                    attributes = pickle_deepcopy(config)
                    attributes[CustomAttributes.RESOURCE_TYPE] = resource_type

                    self.vertices.append(ServerlessBlock(
                        name=f'{resource_type}.{attribute}',
                        config=config,
                        path=file_path,
                        block_type=resource_type,
                        attributes=attributes,
                        id=f"{file_path}:{resource_type}.{attribute}"
                    ))

    def get_resources_types_in_graph(self) -> list[str]:
        # not used
        return []

    @staticmethod
    def update_vertex_config(vertex: _Block, changed_attributes: list[str] | dict[str, Any],
                             has_dynamic_blocks: bool = False) -> None:
        pass

    def update_vertices_configs(self) -> None:
        pass



from aws_cdk import core
from aws_cdk import aws_ec2 as ec2

class MySecurityGroupStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define EC2 Security Group
        security_group = ec2.CfnSecurityGroup(
            self, 'MySecurityGroup',
            group_description='My security group',
            security_group_ingress=[
                {
                    'description': 'False',
                    'ipProtocol': 'tcp',
                    'fromPort': 80,
                    'toPort': 80,
                    'cidrIp': '0.0.0.0/0'
                }
            ],
            # Other properties for your Security Group
        )

app = core.App()
MySecurityGroupStack(app, "MySecurityGroupStack")
app.synth()



class MySecurityGroupStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define EC2 Security Group
        security_group = ec2.CfnSecurityGroup(
            self, 'MySecurityGroup',
            group_description='My security group',
            security_group_egress=[
                {
                    'description': 'False',
                    'ipProtocol': 'tcp',
                    'fromPort': 80,
                    'toPort': 80,
                    'cidrIp': '0.0.0.0/0'
                }
            ],
            # Other properties for your Security Group
        )

app = core.App()
MySecurityGroupStack(app, "MySecurityGroupStack")
app.synth()


class MySecurityGroupIngressStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define EC2 Security Group Ingress
        security_group_ingress = ec2.CfnSecurityGroupIngress(
            self, 'MySecurityGroupIngress',
            group_id='your-security-group-id',  # Replace with your Security Group ID
            ip_protocol='tcp',
            from_port=80,
            to_port=80,
            cidr_ip='0.0.0.0/0',
            # Other properties for your Security Group Ingress
        )

app = core.App()
MySecurityGroupIngressStack(app, "MySecurityGroupIngressStack")
app.synth()

class MySecurityGroupEgressStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define EC2 Security Group Ingress
        security_group_ingress = ec2.CfnSecurityGroupEgress(
            self, 'MySecurityGroupIngress',
            group_id='your-security-group-id',  # Replace with your Security Group ID
            ip_protocol='tcp',
            from_port=80,
            to_port=80,
            cidr_ip='0.0.0.0/0',
            # Other properties for your Security Group Ingress
        )

app = core.App()
MySecurityGroupEgressStack(app, "MySecurityGroupEgressStack")
app.synth()



import unittest

from checkov.terraform.context_parsers.registry import parser_registry
from tests.terraform.context_parsers.mock_context_parser import MockContextParser
import os

mock_definition = (os.path.dirname(os.path.realpath(__file__)) + '/mock_tf_files/mock.tf', {'mock': [
    {
        'mock_type': {
            'mock_name': {
                'value': [
                    'mock_value']}}}
]})


class TestScannerRegistry(unittest.TestCase):

    def test_enrich_definition_block(self):
        mock_parser = MockContextParser()
        parser_registry.register(mock_parser)
        definition_context = parser_registry.enrich_definitions_context(mock_definition)
        self.assertIsNotNone(definition_context[mock_definition[0]]['mock']['mock_type']['mock_name'])


if __name__ == '__main__':
    unittest.main()



import unittest

from checkov.secrets.log_prefix_stripper import (
    strip_log_prefix,
    has_log_prefixes,
    strip_log_prefixes_from_content,
    create_stripped_content,
)
from pathlib import Path

from checkov.secrets.runner import Runner
from detect_secrets.settings import transient_settings


class TestStripLogPrefix(unittest.TestCase):
    """Tests for the strip_log_prefix function."""

    def test_pipe_separated_prefix(self) -> None:
        line = "2026-01-07 09:41:37.553 | DEBUG | crypto      | -----BEGIN RSA PRIVATE KEY-----"
        result = strip_log_prefix(line)
        self.assertEqual(result, "-----BEGIN RSA PRIVATE KEY-----")

    def test_pipe_separated_prefix_with_info(self) -> None:
        line = "2026-01-07 09:41:37.552 | INFO  | crypto      | Decrypting signing key"
        result = strip_log_prefix(line)
        self.assertEqual(result, "Decrypting signing key")

    def test_bracket_style_prefix(self) -> None:
        line = "[2026-01-07 09:41:37] [DEBUG] -----BEGIN RSA PRIVATE KEY-----"
        result = strip_log_prefix(line)
        self.assertEqual(result, "-----BEGIN RSA PRIVATE KEY-----")

    def test_bracket_style_with_module(self) -> None:
        line = "[2026-01-07 09:41:37] [INFO] [crypto] Decrypting signing key"
        result = strip_log_prefix(line)
        self.assertEqual(result, "Decrypting signing key")

    def test_space_separated_prefix(self) -> None:
        line = "2026-01-07 09:41:37.553 DEBUG crypto - -----BEGIN RSA PRIVATE KEY-----"
        result = strip_log_prefix(line)
        self.assertEqual(result, "-----BEGIN RSA PRIVATE KEY-----")

    def test_iso_timestamp_prefix(self) -> None:
        line = "2026-01-07T09:41:37.553Z | INFO  | Starting application"
        result = strip_log_prefix(line)
        self.assertEqual(result, "Starting application")

    def test_time_only_prefix(self) -> None:
        line = "09:41:37.553 | DEBUG | crypto | -----BEGIN RSA PRIVATE KEY-----"
        result = strip_log_prefix(line)
        self.assertEqual(result, "-----BEGIN RSA PRIVATE KEY-----")

    def test_no_prefix(self) -> None:
        line = "-----BEGIN RSA PRIVATE KEY-----"
        result = strip_log_prefix(line)
        self.assertEqual(result, "-----BEGIN RSA PRIVATE KEY-----")

    def test_plain_text_no_prefix(self) -> None:
        line = "This is just a normal line of text"
        result = strip_log_prefix(line)
        self.assertEqual(result, "This is just a normal line of text")

    def test_empty_line(self) -> None:
        result = strip_log_prefix("")
        self.assertEqual(result, "")

    def test_base64_content_preserved(self) -> None:
        line = "2026-01-07 09:41:37.553 | DEBUG | crypto      | MIIEpAIBAAKCAQEAx9TgJ2Zy7KX8rJ3k8PpZrK7aR0L8"
        result = strip_log_prefix(line)
        self.assertEqual(result, "MIIEpAIBAAKCAQEAx9TgJ2Zy7KX8rJ3k8PpZrK7aR0L8")


class TestHasLogPrefixes(unittest.TestCase):
    """Tests for the has_log_prefixes function."""

    def test_log_file_detected(self) -> None:
        content = (
            "2026-01-07 09:41:37.550 | INFO  | main        | Starting application\n"
            "2026-01-07 09:41:37.551 | INFO  | config      | Loading configuration\n"
            "2026-01-07 09:41:37.552 | INFO  | crypto      | Decrypting signing key\n"
            "2026-01-07 09:41:37.553 | DEBUG | crypto      | -----BEGIN RSA PRIVATE KEY-----\n"
            "2026-01-07 09:41:37.553 | DEBUG | crypto      | MIIEpAIBAAKCAQEAx9TgJ2Zy7KX8\n"
        )
        self.assertTrue(has_log_prefixes(content))

    def test_plain_file_not_detected(self) -> None:
        content = (
            "-----BEGIN RSA PRIVATE KEY-----\n"
            "MIIEpAIBAAKCAQEAx9TgJ2Zy7KX8rJ3k8PpZrK7aR0L8\n"
            "-----END RSA PRIVATE KEY-----\n"
        )
        self.assertFalse(has_log_prefixes(content))

    def test_mixed_content_below_threshold(self) -> None:
        content = (
            "This is a normal line\n"
            "Another normal line\n"
            "Yet another normal line\n"
            "2026-01-07 09:41:37.553 | DEBUG | crypto      | one log line\n"
            "More normal content\n"
            "Even more normal content\n"
            "Still normal\n"
            "Normal again\n"
            "Normal once more\n"
            "Final normal line\n"
        )
        self.assertFalse(has_log_prefixes(content))

    def test_empty_content(self) -> None:
        self.assertFalse(has_log_prefixes(""))

    def test_bracket_log_detected(self) -> None:
        content = (
            "[2026-01-07 09:41:37] [INFO] Starting application\n"
            "[2026-01-07 09:41:37] [INFO] Loading configuration\n"
            "[2026-01-07 09:41:37] [DEBUG] Some debug info\n"
        )
        self.assertTrue(has_log_prefixes(content))


class TestStripLogPrefixesFromContent(unittest.TestCase):
    """Tests for the strip_log_prefixes_from_content function."""

    def test_strips_all_prefixes(self) -> None:
        content = (
            "2026-01-07 09:41:37.552 | INFO  | crypto      | Decrypting signing key\n"
            "2026-01-07 09:41:37.553 | DEBUG | crypto      | -----BEGIN RSA PRIVATE KEY-----\n"
            "2026-01-07 09:41:37.553 | DEBUG | crypto      | MIIEpAIBAAKCAQEAx9TgJ2Zy7KX8\n"
            "2026-01-07 09:41:37.553 | DEBUG | crypto      | -----END RSA PRIVATE KEY-----"
        )
        expected = (
            "Decrypting signing key\n"
            "-----BEGIN RSA PRIVATE KEY-----\n"
            "MIIEpAIBAAKCAQEAx9TgJ2Zy7KX8\n"
            "-----END RSA PRIVATE KEY-----"
        )
        result = strip_log_prefixes_from_content(content)
        self.assertEqual(result, expected)


class TestCreateStrippedContent(unittest.TestCase):
    """Tests for the create_stripped_content function."""

    def test_build_log_file(self) -> None:
        test_file = str(Path(__file__).parent / "build_log_prefix" / "build_log_with_private_key.log")
        content = create_stripped_content(test_file)
        self.assertIsNotNone(content)
        self.assertIn("-----BEGIN RSA PRIVATE KEY-----", content)
        # Verify prefixes are removed
        self.assertNotIn("2026-01-07 09:41:37.553 | DEBUG | crypto      | -----BEGIN", content)

    def test_plain_file_not_stripped(self) -> None:
        test_file = str(Path(__file__).parent / "build_log_prefix" / "plain_private_key.txt")
        content = create_stripped_content(test_file)
        self.assertIsNone(content)

    def test_nonexistent_file(self) -> None:
        content = create_stripped_content("/nonexistent/file.log")
        self.assertIsNone(content)

    def test_bracket_log_file(self) -> None:
        test_file = str(Path(__file__).parent / "build_log_prefix" / "bracket_log_with_private_key.log")
        content = create_stripped_content(test_file)
        self.assertIsNotNone(content)
        self.assertIn("-----BEGIN RSA PRIVATE KEY-----", content)
        self.assertNotIn("[2026-01-07 09:41:37]", content)

    def test_putty_key_log_file(self) -> None:
        test_file = str(Path(__file__).parent / "build_log_prefix" / "build_log_with_putty_key.log")
        content = create_stripped_content(test_file)
        self.assertIsNotNone(content)
        self.assertIn("PuTTY-User-Key-File-2: ssh-rsa", content)
        self.assertNotIn("2026-01-07 09:41:37.552 | DEBUG | ssh         | PuTTY", content)


class TestSafeScanWithLogPrefixes(unittest.TestCase):
    """Integration tests verifying _safe_scan detects secrets in build logs."""

    def setUp(self) -> None:
        self.runner = Runner()
        self.plugins_used, self.cleanupFn = self.runner._get_plugins_used()

    def tearDown(self) -> None:
        self.cleanupFn()

    def test_rsa_private_key_in_pipe_log(self) -> None:
        """RSA private key in pipe-separated build log should be detected (CKV_SECRET_13)."""
        with transient_settings({'plugins_used': self.plugins_used}) as settings:
            settings.disable_filters('detect_secrets.filters.heuristic.is_indirect_reference')
            settings.disable_filters('detect_secrets.filters.heuristic.is_potential_uuid')

            _, results = Runner._safe_scan(
                'tests/secrets/build_log_prefix/build_log_with_private_key.log', '.'
            )
        self.assertGreaterEqual(len(results), 1)
        secret_types = {r.type for r in results}
        self.assertIn('Private Key', secret_types)

    def test_rsa_private_key_in_bracket_log(self) -> None:
        """RSA private key in bracket-style build log should be detected (CKV_SECRET_13)."""
        with transient_settings({'plugins_used': self.plugins_used}) as settings:
            settings.disable_filters('detect_secrets.filters.heuristic.is_indirect_reference')
            settings.disable_filters('detect_secrets.filters.heuristic.is_potential_uuid')

            _, results = Runner._safe_scan(
                'tests/secrets/build_log_prefix/bracket_log_with_private_key.log', '.'
            )
        self.assertGreaterEqual(len(results), 1)
        secret_types = {r.type for r in results}
        self.assertIn('Private Key', secret_types)

    def test_putty_key_in_build_log(self) -> None:
        """PuTTY key in build log should be detected (CKV_SECRET_13)."""
        with transient_settings({'plugins_used': self.plugins_used}) as settings:
            settings.disable_filters('detect_secrets.filters.heuristic.is_indirect_reference')
            settings.disable_filters('detect_secrets.filters.heuristic.is_potential_uuid')

            _, results = Runner._safe_scan(
                'tests/secrets/build_log_prefix/build_log_with_putty_key.log', '.'
            )
        self.assertGreaterEqual(len(results), 1)
        secret_types = {r.type for r in results}
        self.assertIn('Private Key', secret_types)

    def test_plain_file_still_detected(self) -> None:
        """Plain file without log prefixes should still detect secrets normally."""
        with transient_settings({'plugins_used': self.plugins_used}) as settings:
            settings.disable_filters('detect_secrets.filters.heuristic.is_indirect_reference')
            settings.disable_filters('detect_secrets.filters.heuristic.is_potential_uuid')

            _, results = Runner._safe_scan(
                'tests/secrets/build_log_prefix/plain_private_key.txt', '.'
            )
        self.assertGreaterEqual(len(results), 1)
        secret_types = {r.type for r in results}
        self.assertIn('Private Key', secret_types)

    def test_mixed_secrets_in_build_log(self) -> None:
        """Both AWS key (single-line) and RSA private key (multiline) should be detected
        in a single build log scan — verifying the file is only scanned once with
        stripped content and both secret types are found."""
        with transient_settings({'plugins_used': self.plugins_used}) as settings:
            settings.disable_filters('detect_secrets.filters.heuristic.is_indirect_reference')
            settings.disable_filters('detect_secrets.filters.heuristic.is_potential_uuid')

            _, results = Runner._safe_scan(
                'tests/secrets/build_log_prefix/build_log_with_mixed_secrets.log', '.'
            )
        secret_types = {r.type for r in results}
        self.assertIn('AWS Access Key', secret_types)
        self.assertIn('Private Key', secret_types)


if __name__ == "__main__":
    unittest.main()



from pathlib import Path

from checkov.common.graph.db_connectors.rustworkx.rustworkx_db_connector import RustworkxConnector
from checkov.common.util.consts import LINE_FIELD_NAMES
from checkov.serverless.graph_builder.graph_to_definitions import convert_graph_vertices_to_definitions
from checkov.serverless.graph_manager import ServerlessGraphManager
from checkov.serverless.utils import get_files_definitions, ServerlessElements

RESOURCES_DIR = Path(__file__).parent / 'resources'

def test_graph_from_file_def_and_graph_def():
    # compare graph created from definition created by file and graph created by definition created from graph
    # given
    test_file = RESOURCES_DIR / "serverless.yaml"
    definitions_from_file, _ = get_files_definitions([str(test_file)])
    graph_manager = ServerlessGraphManager(db_connector=RustworkxConnector())

    local_graph = graph_manager.build_graph_from_definitions(definitions=definitions_from_file, render_variables=False)

    definition_from_graph, _ = convert_graph_vertices_to_definitions(
        vertices=local_graph.vertices,
        root_folder=test_file,
    )
    local_graph_from_new_def = graph_manager.build_graph_from_definitions(definitions=definition_from_graph,
                                                                          render_variables=False)

    # then
    assert len(local_graph.vertices) == len(local_graph_from_new_def.vertices)
    assert len(local_graph.edges) == len(local_graph_from_new_def.edges)

    assert len(local_graph.vertices) == len(local_graph_from_new_def.vertices)
    assert len(local_graph.edges) == len(local_graph_from_new_def.edges)
    assert len(local_graph.vertices_by_block_type[ServerlessElements.PARAMS]) == len(
        local_graph_from_new_def.vertices_by_block_type[ServerlessElements.PARAMS])
    assert len(local_graph.vertices_by_block_type[ServerlessElements.FUNCTIONS]) == len(
        local_graph_from_new_def.vertices_by_block_type[ServerlessElements.FUNCTIONS])
    assert len(local_graph.vertices_by_block_type[ServerlessElements.PROVIDER]) == len(
        local_graph_from_new_def.vertices_by_block_type[ServerlessElements.PROVIDER])
    assert len(local_graph.vertices_by_block_type[ServerlessElements.LAYERS]) == len(
        local_graph_from_new_def.vertices_by_block_type[ServerlessElements.LAYERS])
    assert len(local_graph.vertices_by_block_type[ServerlessElements.CUSTOM]) == len(
        local_graph_from_new_def.vertices_by_block_type[ServerlessElements.CUSTOM])
    assert len(local_graph.vertices_by_block_type[ServerlessElements.PACKAGE]) == len(
        local_graph_from_new_def.vertices_by_block_type[ServerlessElements.PACKAGE])
    assert len(local_graph.vertices_by_block_type[ServerlessElements.PLUGINS]) == len(
        local_graph_from_new_def.vertices_by_block_type[ServerlessElements.PLUGINS])
    assert len(local_graph.vertices_by_block_type[ServerlessElements.SERVICE]) == len(
        local_graph_from_new_def.vertices_by_block_type[ServerlessElements.SERVICE])

    for vertex in local_graph.vertices:
        vertex_file = vertex.path
        vertex_name = vertex.name

        matching_vertex_index = local_graph_from_new_def.vertices_by_path_and_name[(vertex_file, vertex_name)]
        matching_vertex = local_graph_from_new_def.vertices[matching_vertex_index]
        for attribute, value in matching_vertex.config.items():
            assert isinstance(vertex.config[attribute], type(value))
            assert value == vertex.config[attribute]



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.json_doc.base_json_check import BaseJsonCheck
from checkov.json_doc.enums import BlockType


class ValueIsAtLeastTwo(BaseJsonCheck):
    def __init__(self):
        name = "Ensure that an object has a value >= 2"
        id = "CKV_COMPLEX_1"
        categories = [CheckCategories.CONVENTION]
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_entities=["prop_is_array_of_object"],
            block_type=BlockType.ARRAY,
            path="array_of_objects"
        )

    def scan_entity_conf(self, conf, entity_type):
        for obj in conf:
            if obj["value"] < 2:
                return CheckResult.FAILED, obj
        return CheckResult.PASSED


check = ValueIsAtLeastTwo()



from __future__ import annotations
from typing import Any, Tuple

import pytest


@pytest.fixture
def supported_entities() -> Tuple[str, str]:
    return 'jobs', 'stages[].jobs[]'


@pytest.fixture
def definitions() -> dict[str, Any]:
    return {
        '/checkov/tests/azure_pipelines/resources/azure-pipelines.yml': {
            'trigger': ['master'],
            'resources': {
                'repositories': [
                    {
                        'repository': 'AzureDevOps',
                        'type': 'git',
                        'endpoint': 'AzureDevOps',
                        'name': 'AzureDevOps/AzureDevOps',
                        '__startline__': 6,
                        '__endline__': 11
                    }
                ],
                '__startline__': 5,
                '__endline__': 11
            },
            'stages': [
                {
                    'stage': 'Example',
                    'jobs': [
                        {
                            'job': 'FailNoTag',
                            'displayName': 'FailNoTagDisplayName',
                            'pool': {
                                'vmImage': 'ubuntu-18.04',
                                '__startline__': 16,
                                '__endline__': 18
                            },
                            'container': 'ubuntu',
                            'steps': [
                                {
                                    'script': 'printenv',
                                    '__startline__': 21,
                                    '__endline__': 22
                                }
                            ],
                            '__startline__': 14,
                            '__endline__': 22
                        },
                        {
                            'job': 'PassDigest',
                            'pool': {
                                'vmImage': 'ubuntu-18.04',
                                '__startline__': 24,
                                '__endline__': 26
                            },
                            'container': 'ubuntu@sha256:a0a45bd8c6c4acd6967396366f01f2a68f73406327285edc5b7b07cb1cf073db',
                            'steps': [
                                {
                                    'script': 'printenv',
                                    '__startline__': 29,
                                    '__endline__': 31
                                }
                            ],
                            '__startline__': 22,
                            '__endline__': 31
                        }
                    ],
                    '__startline__': 12,
                    '__endline__': 31
                }
            ],
            'jobs': [
                {
                    'job': 'FailTag',
                    'pool': {
                        'vmImage': 'ubuntu-18.04',
                        '__startline__': 34,
                        '__endline__': 36
                    },
                    'container': 'ubuntu:20.04',
                    'steps': [
                        {
                            'script': 'printenv',
                            '__startline__': 39,
                            '__endline__': 39
                        }
                    ],
                    '__startline__': 32,
                    '__endline__': 39
                }
            ],
            '__startline__': 1,
            '__endline__': 39
        }
    }



import unittest
import hcl2

from checkov.terraform.checks.resource.aws.SageMakerInternetAccessDisabled import check
from checkov.common.models.enums import CheckResult


class TestSageMakerInternetAccessDisabled(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
                resource "aws_sagemaker_notebook_instance" "test" {
                  name          = "my-notebook-instance"
                  role_arn      = aws_iam_role.role.arn
                  instance_type = "ml.t2.medium"
                  direct_internet_access = "Enabled"
                                    
                  tags = {
                    Name = "foo"
                  }
                }
        """)
        resource_conf = hcl_res['resource'][0]['aws_sagemaker_notebook_instance']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
                resource "aws_sagemaker_notebook_instance" "test" {
                  name          = "my-notebook-instance"
                  role_arn      = aws_iam_role.role.arn
                  instance_type = "ml.t2.medium"
                  direct_internet_access = "Disabled"
                                  
                  tags = {
                    Name = "foo"
                  }
                }
        """)
        resource_conf = hcl_res['resource'][0]['aws_sagemaker_notebook_instance']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success2(self):
        hcl_res = hcl2.loads("""
                resource "aws_sagemaker_notebook_instance" "test" {
                  name          = "my-notebook-instance"
                  role_arn      = aws_iam_role.role.arn
                  instance_type = "ml.t2.medium"
                                  
                  tags = {
                    Name = "foo"
                  }
                }
        """)
        resource_conf = hcl_res['resource'][0]['aws_sagemaker_notebook_instance']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()
